#!/usr/bin/python
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pagerduty_alert
short_description: Trigger, acknowledge or resolve PagerDuty incidents
description:
  - This module lets you trigger, acknowledge or resolve a PagerDuty incident by sending events.
author:
  - "Amanpreet Singh (@ApsOps)"
  - "Xiao Shen (@xshen1)"
requirements:
  - PagerDuty API access
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - PagerDuty unique subdomain. Obsolete. It is not used with PagerDuty REST v2 API.
  api_key:
    type: str
    description:
      - The pagerduty API key (readonly access), generated on the pagerduty site.
      - Required if O(api_version=v1).
  integration_key:
    type: str
    description:
      - The GUID of one of your 'Generic API' services.
      - This is the 'integration key' listed on a 'Integrations' tab of PagerDuty service.
  service_id:
    type: str
    description:
      - ID of PagerDuty service when incidents are triggered, acknowledged or resolved.
      - Required if O(api_version=v1).
  service_key:
    type: str
    description:
      - The GUID of one of your 'Generic API' services. Obsolete. Please use O(integration_key).
  state:
    type: str
    description:
      - Type of event to be sent.
    required: true
    choices:
      - 'triggered'
      - 'acknowledged'
      - 'resolved'
  api_version:
    type: str
    description:
      - The API version we want to use to run the module.
      - V1 is more limited with option we can provide to trigger incident.
      - V2 has more variables for example, O(severity), O(source), O(custom_details) and so on.
    default: 'v1'
    choices:
      - 'v1'
      - 'v2'
    version_added: 7.4.0
  client:
    type: str
    description:
      - The name of the monitoring client that is triggering this event.
  client_url:
    type: str
    description:
      - The URL of the monitoring client that is triggering this event.
  component:
    type: str
    description:
      - Component of the source machine that is responsible for the event, for example C(mysql) or C(eth0).
    version_added: 7.4.0
  custom_details:
    type: dict
    description:
      - Additional details about the event and affected system.
      - A dictionary with custom keys and values.
    version_added: 7.4.0
  desc:
    type: str
    description:
      - For O(state=triggered) - Required. Short description of the problem that led to this trigger. This field (or a truncated
        version) is used when generating phone calls, SMS messages and alert emails. It also appears on the incidents tables
        in the PagerDuty UI. The maximum length is 1024 characters.
      - For O(state=acknowledged) or O(state=resolved) - Text that appears in the incident's log associated with this event.
    default: Created via Ansible
  incident_class:
    type: str
    description:
      - The class/type of the event, for example C(ping failure) or C(cpu load).
    version_added: 7.4.0
  incident_key:
    type: str
    description:
      - Identifies the incident to which this O(state) should be applied.
      - For O(state=triggered) - If there is no open (in other words unresolved) incident with this key, a new one is created.
        If there is already an open incident with a matching key, this event is appended to that incident's log. The event
        key provides an easy way to 'de-dup' problem reports. If no O(incident_key) is provided, then it is generated by PagerDuty.
      - For O(state=acknowledged) or O(state=resolved) - This should be the incident_key you received back when the incident
        was first opened by a trigger event. Acknowledge events referencing resolved or nonexistent incidents is discarded.
  link_url:
    type: str
    description:
      - Relevant link URL to the alert. For example, the website or the job link.
    version_added: 7.4.0
  link_text:
    type: str
    description:
      - A short description of the O(link_url).
    version_added: 7.4.0
  source:
    type: str
    description:
      - The unique location of the affected system, preferably a hostname or FQDN.
      - Required in case of O(state=trigger) and O(api_version=v2).
    version_added: 7.4.0
  severity:
    type: str
    description:
      - The perceived severity of the status the event is describing with respect to the affected system.
      - Required in case of O(state=trigger) and O(api_version=v2).
    default: 'critical'
    choices:
      - 'critical'
      - 'warning'
      - 'error'
      - 'info'
    version_added: 7.4.0
"""

EXAMPLES = r"""
- name: Trigger an incident with just the basic options
  community.general.pagerduty_alert:
    name: companyabc
    integration_key: xxx
    api_key: yourapikey
    service_id: PDservice
    state: triggered
    desc: problem that led to this trigger

- name: Trigger an incident with more options
  community.general.pagerduty_alert:
    integration_key: xxx
    api_key: yourapikey
    service_id: PDservice
    state: triggered
    desc: problem that led to this trigger
    incident_key: somekey
    client: Sample Monitoring Service
    client_url: http://service.example.com

- name: Acknowledge an incident based on incident_key
  community.general.pagerduty_alert:
    integration_key: xxx
    api_key: yourapikey
    service_id: PDservice
    state: acknowledged
    incident_key: somekey
    desc: "some text for incident's log"

- name: Resolve an incident based on incident_key
  community.general.pagerduty_alert:
    integration_key: xxx
    api_key: yourapikey
    service_id: PDservice
    state: resolved
    incident_key: somekey
    desc: "some text for incident's log"

- name: Trigger an v2 incident with just the basic options
  community.general.pagerduty_alert:
    integration_key: xxx
    api_version: v2
    source: My Ansible Script
    state: triggered
    desc: problem that led to this trigger

- name: Trigger an v2 incident with more options
  community.general.pagerduty_alert:
    integration_key: xxx
    api_version: v2
    source: My Ansible Script
    state: triggered
    desc: problem that led to this trigger
    incident_key: somekey
    client: Sample Monitoring Service
    client_url: http://service.example.com
    component: mysql
    incident_class: ping failure
    link_url: https://pagerduty.com
    link_text: PagerDuty

- name: Acknowledge an incident based on incident_key using v2
  community.general.pagerduty_alert:
    api_version: v2
    integration_key: xxx
    incident_key: somekey
    state: acknowledged

- name: Resolve an incident based on incident_key
  community.general.pagerduty_alert:
    api_version: v2
    integration_key: xxx
    incident_key: somekey
    state: resolved
"""
import json
from datetime import datetime
from urllib.parse import urlencode, urlparse, urlunparse

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def check(module, name, state, service_id, integration_key, api_key, incident_key=None, http_call=fetch_url):
    url = "https://api.pagerduty.com/incidents"
    headers = {
        "Content-type": "application/json",
        "Authorization": f"Token token={api_key}",
        "Accept": "application/vnd.pagerduty+json;version=2",
    }

    params = {"service_ids[]": service_id, "sort_by": "incident_number:desc", "time_zone": "UTC"}
    if incident_key:
        params["incident_key"] = incident_key

    url_parts = list(urlparse(url))
    url_parts[4] = urlencode(params, True)

    url = urlunparse(url_parts)

    response, info = http_call(module, url, method="get", headers=headers)

    if info["status"] != 200:
        module.fail_json(msg=f"failed to check current incident status.Reason: {info['msg']}")

    incidents = json.loads(response.read())["incidents"]
    msg = "No corresponding incident"

    if len(incidents) == 0:
        if state in ("acknowledged", "resolved"):
            return msg, False
        return msg, True
    elif state != incidents[0]["status"]:
        return incidents[0], True

    return incidents[0], False


def send_event_v1(module, service_key, event_type, desc, incident_key=None, client=None, client_url=None):
    url = "https://events.pagerduty.com/generic/2010-04-15/create_event.json"
    headers = {"Content-type": "application/json"}

    data = {
        "service_key": service_key,
        "event_type": event_type,
        "incident_key": incident_key,
        "description": desc,
        "client": client,
        "client_url": client_url,
    }

    response, info = fetch_url(module, url, method="post", headers=headers, data=json.dumps(data))
    if info["status"] != 200:
        module.fail_json(msg=f"failed to {event_type}. Reason: {info['msg']}")
    json_out = json.loads(response.read())
    return json_out


def send_event_v2(module, service_key, event_type, payload, link, incident_key=None, client=None, client_url=None):
    url = "https://events.pagerduty.com/v2/enqueue"
    headers = {"Content-type": "application/json"}
    data = {
        "routing_key": service_key,
        "event_action": event_type,
        "payload": payload,
        "client": client,
        "client_url": client_url,
    }
    if link:
        data["links"] = [link]
    if incident_key:
        data["dedup_key"] = incident_key
    if event_type != "trigger":
        data.pop("payload")
    response, info = fetch_url(module, url, method="post", headers=headers, data=json.dumps(data))
    if info["status"] != 202:
        module.fail_json(msg=f"failed to {event_type}. Reason: {info['msg']}")
    json_out = json.loads(response.read())
    return json_out, True


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(),
            api_key=dict(no_log=True),
            integration_key=dict(no_log=True),
            service_id=dict(),
            service_key=dict(no_log=True),
            state=dict(required=True, choices=["triggered", "acknowledged", "resolved"]),
            api_version=dict(type="str", default="v1", choices=["v1", "v2"]),
            client=dict(),
            client_url=dict(),
            component=dict(),
            custom_details=dict(type="dict"),
            desc=dict(default="Created via Ansible"),
            incident_class=dict(),
            incident_key=dict(no_log=False),
            link_url=dict(),
            link_text=dict(),
            source=dict(),
            severity=dict(default="critical", choices=["critical", "warning", "error", "info"]),
        ),
        required_if=[
            ("api_version", "v1", ["service_id", "api_key"]),
            ("state", "acknowledged", ["incident_key"]),
            ("state", "resolved", ["incident_key"]),
        ],
        required_one_of=[("service_key", "integration_key")],
        supports_check_mode=True,
    )

    name = module.params["name"]
    service_id = module.params.get("service_id")
    integration_key = module.params.get("integration_key")
    service_key = module.params.get("service_key")
    api_key = module.params.get("api_key")
    state = module.params.get("state")
    client = module.params.get("client")
    client_url = module.params.get("client_url")
    desc = module.params.get("desc")
    incident_key = module.params.get("incident_key")
    payload = {
        "summary": desc,
        "source": module.params.get("source"),
        "timestamp": datetime.now().isoformat(),
        "severity": module.params.get("severity"),
        "component": module.params.get("component"),
        "class": module.params.get("incident_class"),
        "custom_details": module.params.get("custom_details"),
    }
    link = {}
    if module.params.get("link_url"):
        link["href"] = module.params.get("link_url")
        if module.params.get("link_text"):
            link["text"] = module.params.get("link_text")
    if integration_key is None:
        integration_key = service_key
        module.warn('"service_key" is obsolete parameter and will be removed. Please, use "integration_key" instead')

    state_event_dict = {
        "triggered": "trigger",
        "acknowledged": "acknowledge",
        "resolved": "resolve",
    }

    event_type = state_event_dict[state]
    if module.params.get("api_version") == "v1":
        out, changed = check(module, name, state, service_id, integration_key, api_key, incident_key)
        if not module.check_mode and changed is True:
            out = send_event_v1(module, integration_key, event_type, desc, incident_key, client, client_url)
    else:
        changed = True
        if event_type == "trigger" and not payload["source"]:
            module.fail_json(msg='"service" is a required variable for v2 api endpoint.')
        out, changed = send_event_v2(
            module,
            integration_key,
            event_type,
            payload,
            link,
            incident_key,
            client,
            client_url,
        )

    module.exit_json(result=out, changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2020, Andrew Klaus <andrewklaus@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sysupgrade
short_description: Manage OpenBSD system upgrades
version_added: 1.1.0
description:
  - Manage OpenBSD system upgrades using C(sysupgrade).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  snapshot:
    description:
      - Apply the latest snapshot.
      - Otherwise release is applied.
    default: false
    type: bool
  force:
    description:
      - Force upgrade (for snapshots only).
    default: false
    type: bool
  keep_files:
    description:
      - Keep the files under C(/home/_sysupgrade).
      - By default, the files are deleted after the upgrade.
    default: false
    type: bool
  fetch_only:
    description:
      - Fetch and verify files and create C(/bsd.upgrade) but do not reboot.
      - Set to V(false) if you want C(sysupgrade) to reboot. This causes the module to fail. See the examples.
    default: true
    type: bool
  installurl:
    description:
      - OpenBSD mirror top-level URL for fetching an upgrade.
      - By default, the mirror URL is pulled from C(/etc/installurl).
    type: str
author:
  - Andrew Klaus (@precurse)
"""

EXAMPLES = r"""
- name: Upgrade to latest release
  community.general.sysupgrade:
  register: sysupgrade

- name: Upgrade to latest snapshot
  community.general.sysupgrade:
    snapshot: true
    installurl: https://cloudflare.cdn.openbsd.org/pub/OpenBSD
  register: sysupgrade

- name: Reboot to apply upgrade if needed
  ansible.builtin.reboot:
  when: sysupgrade.changed

# Note: Ansible will error when running this way due to how
#   the reboot is forcefully handled by sysupgrade:

- name: Have sysupgrade automatically reboot
  community.general.sysupgrade:
    fetch_only: false
  ignore_errors: true
"""


from ansible.module_utils.basic import AnsibleModule


def sysupgrade_run(module):
    sysupgrade_bin = module.get_bin_path("/usr/sbin/sysupgrade", required=True)
    cmd = [sysupgrade_bin]
    changed = False

    # Setup command flags
    if module.params["snapshot"]:
        run_flag = ["-s"]
        if module.params["force"]:
            # Force only applies to snapshots
            run_flag.append("-f")
    else:
        # release flag
        run_flag = ["-r"]

    if module.params["keep_files"]:
        run_flag.append("-k")

    if module.params["fetch_only"]:
        run_flag.append("-n")

    # installurl must be the last argument
    if module.params["installurl"]:
        run_flag.append(module.params["installurl"])

    rc, out, err = module.run_command(cmd + run_flag)

    if rc != 0:
        module.fail_json(msg=f"Command {cmd} failed rc={rc}, out={out}, err={err}")
    elif out.lower().find("already on latest snapshot") >= 0:
        changed = False
    elif out.lower().find("upgrade on next reboot") >= 0:
        changed = True

    return dict(
        changed=changed,
        rc=rc,
        stderr=err,
        stdout=out,
    )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            snapshot=dict(type="bool", default=False),
            fetch_only=dict(type="bool", default=True),
            force=dict(type="bool", default=False),
            keep_files=dict(type="bool", default=False),
            installurl=dict(type="str"),
        ),
        supports_check_mode=False,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
    return_dict = sysupgrade_run(module)
    module.exit_json(**return_dict)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_location_scanning_receivers_info
short_description: Information module for organizations _wireless _location _scanning
  _receivers
description:
  - Information module for Organizations Wireless Location Scanning Receivers Info.
  - Get all organizations _wireless _location _scanning _receivers.
  - Return scanning API receivers.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Wireless Location Scanning Receivers Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - Information module for Organizations Wireless Location Scanning Receivers Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 250. Default is 50.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Wireless Location Scanning Receivers Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Wireless Location Scanning Receivers Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - Information module for Organizations Wireless Location Scanning Receivers Info.
      - NetworkIds query parameter. Optional parameter to filter scanning API receivers
        by network ID.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessLocationScanningReceivers
    description: Complete reference of the getOrganizationWirelessLocationScanningReceivers
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-location-scanning-receivers
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_location_scanning_receivers,
  - Paths used are
    get /organizations/{organizationId}/wireless/location/scanning/receivers,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _location _scanning _receivers
  cisco.meraki.organizations_wireless_location_scanning_receivers_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "network": {
            "id": "string",
            "name": "string"
          },
          "receiverId": "string",
          "url": "string",
          "version": "string",
          "radio": {
            "type": "string"
          }
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2016, Adfinis SyGroup AG
# @keachi
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: udm_dns_zone
author:
  - keachi (@keachi)
short_description: Manage DNS zones on a univention corporate server
description:
  - This module allows to manage DNS zones on a univention corporate server (UCS). It uses the Python API of the UCS to create
    a new object or edit it.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
options:
  state:
    type: str
    default: "present"
    choices: [present, absent]
    description:
      - Whether the DNS zone is present or not.
  type:
    type: str
    required: true
    description:
      - Define if the zone is a forward or reverse DNS zone.
      - 'The available choices are: V(forward_zone), V(reverse_zone).'
  zone:
    type: str
    required: true
    description:
      - DNS zone name, for example V(example.com).
    aliases: [name]
  nameserver:
    type: list
    elements: str
    default: []
    description:
      - List of appropriate name servers. Required if O(state=present).
  interfaces:
    type: list
    elements: str
    default: []
    description:
      - List of interface IP addresses, on which the server should response this zone. Required if O(state=present).
  refresh:
    type: int
    default: 3600
    description:
      - Interval before the zone should be refreshed.
  retry:
    type: int
    default: 1800
    description:
      - Interval that should elapse before a failed refresh should be retried.
  expire:
    type: int
    default: 604800
    description:
      - Specifies the upper limit on the time interval that can elapse before the zone is no longer authoritative.
  ttl:
    type: int
    default: 600
    description:
      - Minimum TTL field that should be exported with any RR from this zone.
  contact:
    type: str
    default: ''
    description:
      - Contact person in the SOA record.
  mx:
    type: list
    elements: str
    default: []
    description:
      - List of MX servers. (Must declared as A or AAAA records).
"""


EXAMPLES = r"""
- name: Create a DNS zone on a UCS
  community.general.udm_dns_zone:
    zone: example.com
    type: forward_zone
    nameserver:
      - ucs.example.com
    interfaces:
      - 192.0.2.1
"""


RETURN = """#"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._univention_umc import (
    base_dn,
    ldap_search,
    umc_module_for_add,
    umc_module_for_edit,
)


def convert_time(time):
    """Convert a time in seconds into the biggest unit"""
    units = [
        (24 * 60 * 60, "days"),
        (60 * 60, "hours"),
        (60, "minutes"),
        (1, "seconds"),
    ]

    if time == 0:
        return ("0", "seconds")
    for unit in units:
        if time >= unit[0]:
            return (f"{time // unit[0]}", unit[1])


def main():
    module = AnsibleModule(
        argument_spec=dict(
            type=dict(required=True, type="str"),
            zone=dict(required=True, aliases=["name"], type="str"),
            nameserver=dict(default=[], type="list", elements="str"),
            interfaces=dict(default=[], type="list", elements="str"),
            refresh=dict(default=3600, type="int"),
            retry=dict(default=1800, type="int"),
            expire=dict(default=604800, type="int"),
            ttl=dict(default=600, type="int"),
            contact=dict(default="", type="str"),
            mx=dict(default=[], type="list", elements="str"),
            state=dict(default="present", choices=["present", "absent"], type="str"),
        ),
        supports_check_mode=True,
        required_if=([("state", "present", ["nameserver", "interfaces"])]),
    )
    type = module.params["type"]
    zone = module.params["zone"]
    nameserver = module.params["nameserver"]
    interfaces = module.params["interfaces"]
    refresh = module.params["refresh"]
    retry = module.params["retry"]
    expire = module.params["expire"]
    ttl = module.params["ttl"]
    contact = module.params["contact"]
    mx = module.params["mx"]
    state = module.params["state"]
    changed = False
    diff = None

    obj = list(ldap_search(f"(&(objectClass=dNSZone)(zoneName={zone}))", attr=["dNSZone"]))

    exists = bool(len(obj))
    container = f"cn=dns,{base_dn()}"
    dn = f"zoneName={zone},{container}"
    if contact == "":
        contact = f"root@{zone}."

    if state == "present":
        try:
            if not exists:
                obj = umc_module_for_add(f"dns/{type}", container)
            else:
                obj = umc_module_for_edit(f"dns/{type}", dn)
            obj["zone"] = zone
            obj["nameserver"] = nameserver
            obj["a"] = interfaces
            obj["refresh"] = convert_time(refresh)
            obj["retry"] = convert_time(retry)
            obj["expire"] = convert_time(expire)
            obj["ttl"] = convert_time(ttl)
            obj["contact"] = contact
            obj["mx"] = mx
            diff = obj.diff()
            if exists:
                for k in obj.keys():
                    if obj.hasChanged(k):
                        changed = True
            else:
                changed = True
            if not module.check_mode:
                if not exists:
                    obj.create()
                elif changed:
                    obj.modify()
        except Exception as e:
            module.fail_json(msg=f"Creating/editing dns zone {zone} failed: {e}")

    if state == "absent" and exists:
        try:
            obj = umc_module_for_edit(f"dns/{type}", dn)
            if not module.check_mode:
                obj.remove()
            changed = True
        except Exception as e:
            module.fail_json(msg=f"Removing dns zone {zone} failed: {e}")

    module.exit_json(changed=changed, diff=diff, zone=zone)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Johannes Brunswicker <johannes.brunswicker@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: utm_aaa_group

author:
  - Johannes Brunswicker (@MatrixCrawler)

short_description: Create, update or destroy an aaa group object in Sophos UTM

description:
  - Create, update or destroy an aaa group object in Sophos UTM.
  - This module needs to have the REST Ability of the UTM to be activated.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  name:
    description:
      - The name of the object that identifies the entry.
    type: str
    required: true
  adirectory_groups:
    description:
      - List of adirectory group strings.
    type: list
    elements: str
    default: []
  adirectory_groups_sids:
    description:
      - Dictionary of group sids.
    type: dict
    default: {}
  backend_match:
    description:
      - The backend for the group.
    type: str
    choices:
      - none
      - adirectory
      - edirectory
      - radius
      - tacacs
      - ldap
    default: none
  comment:
    description:
      - Comment that describes the AAA group.
    type: str
    default: ''
  dynamic:
    description:
      - Group type. Is static if none is selected.
    type: str
    default: none
    choices:
      - none
      - ipsec_dn
      - directory_groups
  edirectory_groups:
    description:
      - List of edirectory group strings.
    type: list
    elements: str
    default: []
  ipsec_dn:
    description:
      - The ipsec dn string.
    type: str
    default: ''
  ldap_attribute:
    description:
      - The ldap attribute to check against.
    type: str
    default: ''
  ldap_attribute_value:
    description:
      - The ldap attribute value to check against.
    type: str
    default: ''
  members:
    description:
      - A list of user ref names (aaa/user).
    type: list
    elements: str
    default: []
  network:
    description:
      - The network reference name. The objects contains the known IP addresses for the authentication object (network/aaa).
    type: str
    default: ""
  radius_groups:
    description:
      - A list of radius group strings.
    type: list
    elements: str
    default: []
  tacacs_groups:
    description:
      - A list of tacacs group strings.
    type: list
    elements: str
    default: []

extends_documentation_fragment:
  - community.general._utm
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create UTM aaa_group
  community.general.utm_aaa_group:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestAAAGroupEntry
    backend_match: ldap
    dynamic: directory_groups
    ldap_attributes: memberof
    ldap_attributes_value: "cn=groupname,ou=Groups,dc=mydomain,dc=com"
    network: REF_OBJECT_STRING
    state: present

- name: Remove UTM aaa_group
  community.general.utm_aaa_group:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestAAAGroupEntry
    state: absent
"""

RETURN = r"""
result:
  description: The utm object that was created.
  returned: success
  type: complex
  contains:
    _ref:
      description: The reference name of the object.
      type: str
    _locked:
      description: Whether or not the object is currently locked.
      type: bool
    _type:
      description: The type of the object.
      type: str
    name:
      description: The name of the object.
      type: str
    adirectory_groups:
      description: List of Active Directory Groups.
      type: str
    adirectory_groups_sids:
      description: List of Active Directory Groups SIDS.
      type: list
    backend_match:
      description: The backend to use.
      type: str
    comment:
      description: The comment string.
      type: str
    dynamic:
      description: Whether the group match is ipsec_dn or directory_group.
      type: str
    edirectory_groups:
      description: List of eDirectory Groups.
      type: str
    ipsec_dn:
      description: Ipsec_dn identifier to match.
      type: str
    ldap_attribute:
      description: The LDAP Attribute to match against.
      type: str
    ldap_attribute_value:
      description: The LDAP Attribute Value to match against.
      type: str
    members:
      description: List of member identifiers of the group.
      type: list
    network:
      description: The identifier of the network (network/aaa).
      type: str
    radius_group:
      description: The radius group identifier.
      type: str
    tacacs_group:
      description: The tacacs group identifier.
      type: str
"""

from ansible_collections.community.general.plugins.module_utils._utm_utils import UTM, UTMModule


def main():
    endpoint = "aaa/group"
    key_to_check_for_changes = [
        "comment",
        "adirectory_groups",
        "adirectory_groups_sids",
        "backend_match",
        "dynamic",
        "edirectory_groups",
        "ipsec_dn",
        "ldap_attribute",
        "ldap_attribute_value",
        "members",
        "network",
        "radius_groups",
        "tacacs_groups",
    ]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            adirectory_groups=dict(type="list", elements="str", default=[]),
            adirectory_groups_sids=dict(type="dict", default={}),
            backend_match=dict(
                type="str", default="none", choices=["none", "adirectory", "edirectory", "radius", "tacacs", "ldap"]
            ),
            comment=dict(type="str", default=""),
            dynamic=dict(type="str", default="none", choices=["none", "ipsec_dn", "directory_groups"]),
            edirectory_groups=dict(type="list", elements="str", default=[]),
            ipsec_dn=dict(type="str", default=""),
            ldap_attribute=dict(type="str", default=""),
            ldap_attribute_value=dict(type="str", default=""),
            members=dict(type="list", elements="str", default=[]),
            network=dict(type="str", default=""),
            radius_groups=dict(type="list", elements="str", default=[]),
            tacacs_groups=dict(type="list", elements="str", default=[]),
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_volume_info
short_description: Gather information about the Scaleway volumes available
description:
  - Gather information about the Scaleway volumes available.
author:
  - "Yanis Guenane (@Spredzy)"
  - "Remy Leone (@remyleone)"
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway
  - community.general._attributes.info_module

attributes:
  action_group:
    version_added: 11.3.0

options:
  region:
    type: str
    description:
      - Scaleway region to use (for example C(par1)).
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - ams2
      - ams3
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - par3
      - waw1
      - EMEA-PL-WAW1
      - waw2
      - waw3
"""

EXAMPLES = r"""
- name: Gather Scaleway volumes information
  community.general.scaleway_volume_info:
    region: par1
  register: result

- ansible.builtin.debug:
    msg: "{{ result.scaleway_volume_info }}"
"""

RETURN = r"""
scaleway_volume_info:
  description:
    - Response from Scaleway API.
    - 'For more details please refer to: U(https://developers.scaleway.com/en/products/instance/api/).'
  returned: success
  type: list
  elements: dict
  sample:
    [
      {
        "creation_date": "2018-08-14T20:56:24.949660+00:00",
        "export_uri": null,
        "id": "b8d51a06-daeb-4fef-9539-a8aea016c1ba",
        "modification_date": "2018-08-14T20:56:24.949660+00:00",
        "name": "test-volume",
        "organization": "3f709602-5e6c-4619-b80c-e841c89734af",
        "server": null,
        "size": 50000000000,
        "state": "available",
        "volume_type": "l_ssd"
      }
    ]
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_LOCATION,
    Scaleway,
    ScalewayException,
    scaleway_argument_spec,
)


class ScalewayVolumeInfo(Scaleway):
    def __init__(self, module):
        super().__init__(module)
        self.name = "volumes"

        region = module.params["region"]
        self.module.params["api_url"] = SCALEWAY_LOCATION[region]["api_endpoint"]


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        dict(
            region=dict(required=True, choices=list(SCALEWAY_LOCATION.keys())),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        module.exit_json(scaleway_volume_info=ScalewayVolumeInfo(module).get_resources())
    except ScalewayException as exc:
        module.fail_json(msg=exc.message)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015-2023, Vlad Glagolev <scm@vaygr.net>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sorcery
short_description: Package manager for Source Mage GNU/Linux
description:
  - Manages "spells" on Source Mage GNU/Linux using I(sorcery) toolchain.
author: "Vlad Glagolev (@vaygr)"
notes:
  - When all three components are selected, the update goes by the sequence -- Sorcery -> Grimoire(s) -> Spell(s); you cannot
    override it.
  - Grimoire handling is supported since community.general 7.3.0.
requirements:
  - bash
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the spell or grimoire.
      - Multiple names can be given, separated by commas.
      - Special value V(*) in conjunction with states V(latest) or V(rebuild) updates or rebuilds the whole system respectively.
      - The alias O(grimoire) was added in community.general 7.3.0.
    aliases: ["spell", "grimoire"]
    type: list
    elements: str

  repository:
    description:
      - Repository location.
      - If specified, O(name) represents grimoire(s) instead of spell(s).
      - Special value V(*) pulls grimoire from the official location.
      - Only single item in O(name) in conjunction with V(*) can be used.
      - O(state=absent) must be used with a special value V(*).
    type: str
    version_added: 7.3.0

  state:
    description:
      - Whether to cast, dispel or rebuild a package.
      - State V(cast) is an equivalent of V(present), not V(latest).
      - State V(rebuild) implies cast of all specified spells, not only those existed before.
    choices: ["present", "latest", "absent", "cast", "dispelled", "rebuild"]
    default: "present"
    type: str

  depends:
    description:
      - Comma-separated list of _optional_ dependencies to build a spell (or make sure it is built) with; use V(+)/V(-) in
        front of dependency to turn it on/off (V(+) is optional though).
      - This option is ignored if O(name) parameter is equal to V(*) or contains more than one spell.
      - Providers must be supplied in the form recognized by Sorcery, for example 'V(openssl(SSL\))'.
    type: str

  update:
    description:
      - Whether or not to update sorcery scripts at the very first stage.
    type: bool
    default: false

  update_cache:
    description:
      - Whether or not to update grimoire collection before casting spells.
    type: bool
    default: false
    aliases: ["update_codex"]

  cache_valid_time:
    description:
      - Time in seconds to invalidate grimoire collection on update.
      - Especially useful for SCM and rsync grimoires.
      - Makes sense only in pair with O(update_cache).
    type: int
    default: 0
"""


EXAMPLES = r"""
- name: Make sure spell foo is installed
  community.general.sorcery:
    spell: foo
    state: present

- name: Make sure spells foo, bar and baz are removed
  community.general.sorcery:
    spell: foo,bar,baz
    state: absent

- name: Make sure spell foo with dependencies bar and baz is installed
  community.general.sorcery:
    spell: foo
    depends: bar,baz
    state: present

- name: Make sure spell foo with bar and without baz dependencies is installed
  community.general.sorcery:
    spell: foo
    depends: +bar,-baz
    state: present

- name: Make sure spell foo with libressl (providing SSL) dependency is installed
  community.general.sorcery:
    spell: foo
    depends: libressl(SSL)
    state: present

- name: Make sure spells with/without required dependencies (if any) are installed
  community.general.sorcery:
    name: "{{ item.spell }}"
    depends: "{{ item.depends | default(None) }}"
    state: present
  loop:
    - {spell: 'vifm', depends: '+file,-gtk+2'}
    - {spell: 'fwknop', depends: 'gpgme'}
    - {spell: 'pv,tnftp,tor'}

- name: Install the latest version of spell foo using regular glossary
  community.general.sorcery:
    name: foo
    state: latest

- name: Rebuild spell foo
  community.general.sorcery:
    spell: foo
    state: rebuild

- name: Rebuild the whole system, but update Sorcery and Codex first
  community.general.sorcery:
    spell: '*'
    state: rebuild
    update: true
    update_cache: true

- name: Refresh the grimoire collection if it is 1 day old using native sorcerous alias
  community.general.sorcery:
    update_codex: true
    cache_valid_time: 86400

- name: Make sure stable grimoire is present
  community.general.sorcery:
    name: stable
    repository: '*'
    state: present

- name: Make sure binary and stable-rc grimoires are removed
  community.general.sorcery:
    grimoire: binary,stable-rc
    repository: '*'
    state: absent

- name: Make sure games grimoire is pulled from rsync
  community.general.sorcery:
    grimoire: games
    repository: "rsync://download.sourcemage.org::codex/games"
    state: present

- name: Make sure a specific branch of stable grimoire is pulled from git
  community.general.sorcery:
    grimoire: stable.git
    repository: "git://download.sourcemage.org/smgl/grimoire.git:stable.git:stable-0.62"
    state: present

- name: Update only Sorcery itself
  community.general.sorcery:
    update: true
"""


RETURN = r"""
"""


import datetime
import fileinput
import os
import re
import shutil
import sys

from ansible.module_utils.basic import AnsibleModule

# auto-filled at module init
SORCERY = {"sorcery": None, "scribe": None, "cast": None, "dispel": None, "gaze": None}

SORCERY_LOG_DIR = "/var/log/sorcery"
SORCERY_STATE_DIR = "/var/state/sorcery"

NA = "N/A"


def get_sorcery_ver(module):
    """Get Sorcery version."""

    cmd_sorcery = f"{SORCERY['sorcery']} --version"

    rc, stdout, stderr = module.run_command(cmd_sorcery)

    if rc != 0 or not stdout:
        module.fail_json(msg="unable to get Sorcery version")

    return stdout.strip()


def codex_fresh(codex, module):
    """Check if grimoire collection is fresh enough."""

    if not module.params["cache_valid_time"]:
        return False

    timedelta = datetime.timedelta(seconds=module.params["cache_valid_time"])

    for grimoire in codex:
        lastupdate_path = os.path.join(SORCERY_STATE_DIR, f"{grimoire}.lastupdate")

        try:
            mtime = os.stat(lastupdate_path).st_mtime
        except Exception:
            return False

        lastupdate_ts = datetime.datetime.fromtimestamp(mtime)

        # if any grimoire is not fresh, we invalidate the Codex
        if lastupdate_ts + timedelta < datetime.datetime.now():
            return False

    return True


def codex_list(module, skip_new=False):
    """List valid grimoire collection."""

    params = module.params

    codex = {}

    cmd_scribe = f"{SORCERY['scribe']} index"

    rc, stdout, stderr = module.run_command(cmd_scribe)

    if rc != 0:
        module.fail_json(msg="unable to list grimoire collection, fix your Codex")

    rex = re.compile(r"^\s*\[\d+\] : (?P<grim>[\w\-+.]+) : [\w\-+./]+(?: : (?P<ver>[\w\-+.]+))?\s*$")

    # drop 4-line header and empty trailing line
    for line in stdout.splitlines()[4:-1]:
        match = rex.match(line)

        if match:
            codex[match.group("grim")] = match.group("ver")

    # return only specified grimoires unless requested to skip new
    if params["repository"] and not skip_new:
        codex = {x: codex.get(x, NA) for x in params["name"]}

    if not codex:
        module.fail_json(msg="no grimoires to operate on; add at least one")

    return codex


def update_sorcery(module):
    """Update sorcery scripts.

    This runs 'sorcery update' ('sorcery -u'). Check mode always returns a
    positive change value.

    """

    changed = False

    if module.check_mode:
        return (True, "would have updated Sorcery")
    else:
        sorcery_ver = get_sorcery_ver(module)

        cmd_sorcery = f"{SORCERY['sorcery']} update"

        rc, stdout, stderr = module.run_command(cmd_sorcery)

        if rc != 0:
            module.fail_json(msg=f"unable to update Sorcery: {stdout}")

        if sorcery_ver != get_sorcery_ver(module):
            changed = True

        return (changed, "successfully updated Sorcery")


def update_codex(module):
    """Update grimoire collections.

    This runs 'scribe update'. Check mode always returns a positive change
    value when 'cache_valid_time' is used.

    """

    params = module.params

    changed = False

    codex = codex_list(module)
    fresh = codex_fresh(codex, module)

    if module.check_mode:
        if not fresh:
            changed = True

        return (changed, "would have updated Codex")
    else:
        if not fresh:
            # SILENT is required as a workaround for query() in libgpg
            module.run_command_environ_update.update(dict(SILENT="1"))

            cmd_scribe = f"{SORCERY['scribe']} update"

            if params["repository"]:
                cmd_scribe += f" {' '.join(codex.keys())}"

            rc, stdout, stderr = module.run_command(cmd_scribe)

            if rc != 0:
                module.fail_json(msg=f"unable to update Codex: {stdout}")

            if codex != codex_list(module):
                changed = True

        return (changed, "successfully updated Codex")


def match_depends(module):
    """Check for matching dependencies.

    This inspects spell's dependencies with the desired states and returns
    'False' if a recast is needed to match them. It also adds required lines
    to the system-wide depends file for proper recast procedure.

    """

    params = module.params
    spells = params["name"]

    depends = {}

    depends_ok = True

    if len(spells) > 1 or not params["depends"]:
        return depends_ok

    spell = spells[0]

    if module.check_mode:
        sorcery_depends_orig = os.path.join(SORCERY_STATE_DIR, "depends")
        sorcery_depends = os.path.join(SORCERY_STATE_DIR, "depends.check")

        try:
            shutil.copy2(sorcery_depends_orig, sorcery_depends)
        except OSError:
            module.fail_json(msg="failed to copy depends.check file")
    else:
        sorcery_depends = os.path.join(SORCERY_STATE_DIR, "depends")

    rex = re.compile(r"^(?P<status>\+?|\-){1}(?P<depend>[a-z0-9]+[a-z0-9_\-\+\.]*(\([A-Z0-9_\-\+\.]+\))*)$")

    for d in params["depends"].split(","):
        match = rex.match(d)

        if not match:
            module.fail_json(msg=f"wrong depends line for spell '{spell}'")

        # normalize status
        if not match.group("status") or match.group("status") == "+":
            status = "on"
        else:
            status = "off"

        depends[match.group("depend")] = status

    # drop providers spec
    depends_list = [s.split("(")[0] for s in depends]

    cmd_gaze = f"{SORCERY['gaze']} -q version {' '.join(depends_list)}"

    rc, stdout, stderr = module.run_command(cmd_gaze)

    if rc != 0:
        module.fail_json(msg=f"wrong dependencies for spell '{spell}'")

    fi = fileinput.input(sorcery_depends, inplace=True)

    try:
        try:
            for line in fi:
                if line.startswith(f"{spell}:"):
                    match = None

                    for d in depends:
                        # when local status is 'off' and dependency is provider,
                        # use only provider value
                        d_offset = d.find("(")

                        if d_offset == -1:
                            d_p = ""
                        else:
                            d_p = re.escape(d[d_offset:])

                        # .escape() is needed mostly for the spells like 'libsigc++'
                        rex = re.compile(f"{re.escape(spell)}:(?:{re.escape(d)}|{d_p}):(?P<lstatus>on|off):optional:")

                        match = rex.match(line)

                        # we matched the line "spell:dependency:on|off:optional:"
                        if match:
                            # if we also matched the local status, mark dependency
                            # as empty and put it back into depends file
                            if match.group("lstatus") == depends[d]:
                                depends[d] = None

                                sys.stdout.write(line)

                            # status is not that we need, so keep this dependency
                            # in the list for further reverse switching;
                            # stop and process the next line in both cases
                            break

                    if not match:
                        sys.stdout.write(line)
                else:
                    sys.stdout.write(line)
        except OSError:
            module.fail_json(msg="I/O error on the depends file")
    finally:
        fi.close()

    depends_new = [v for v in depends if depends[v]]

    if depends_new:
        try:
            with open(sorcery_depends, "a") as fl:
                for k in depends_new:
                    fl.write(f"{spell}:{k}:{depends[k]}:optional::\n")
        except OSError:
            module.fail_json(msg="I/O error on the depends file")

        depends_ok = False

    if module.check_mode:
        try:
            os.remove(sorcery_depends)
        except OSError:
            module.fail_json(msg="failed to clean up depends.backup file")

    return depends_ok


def manage_grimoires(module):
    """Add or remove grimoires."""

    params = module.params
    grimoires = params["name"]
    url = params["repository"]

    codex = codex_list(module, True)

    if url == "*":
        if params["state"] in ("present", "latest", "absent"):
            if params["state"] == "absent":
                action = "remove"
                todo = set(grimoires) & set(codex)
            else:
                action = "add"
                todo = set(grimoires) - set(codex)

            if not todo:
                return (False, f"all grimoire(s) are already {action[:5]}ed")

            if module.check_mode:
                return (True, f"would have {action[:5]}ed grimoire(s)")

            cmd_scribe = f"{SORCERY['scribe']} {action} {' '.join(todo)}"

            rc, stdout, stderr = module.run_command(cmd_scribe)

            if rc != 0:
                module.fail_json(msg=f"failed to {action} one or more grimoire(s): {stdout}")

            return (True, f"successfully {action[:5]}ed one or more grimoire(s)")
        else:
            module.fail_json(msg="unsupported operation on '*' repository value")
    else:
        if params["state"] in ("present", "latest"):
            if len(grimoires) > 1:
                module.fail_json(msg="using multiple items with repository is invalid")

            grimoire = grimoires[0]

            if grimoire in codex:
                return (False, f"grimoire {grimoire} already exists")

            if module.check_mode:
                return (True, f"would have added grimoire {grimoire} from {url}")

            cmd_scribe = f"{SORCERY['scribe']} add {grimoire} from {url}"

            rc, stdout, stderr = module.run_command(cmd_scribe)

            if rc != 0:
                module.fail_json(msg=f"failed to add grimoire {grimoire} from {url}: {stdout}")

            return (True, f"successfully added grimoire {grimoire} from {url}")
        else:
            module.fail_json(msg="unsupported operation on repository value")


def manage_spells(module):
    """Cast or dispel spells.

    This manages the whole system ('*'), list or a single spell. Command 'cast'
    is used to install or rebuild spells, while 'dispel' takes care of theirs
    removal from the system.

    """

    params = module.params
    spells = params["name"]

    sorcery_queue = os.path.join(SORCERY_LOG_DIR, "queue/install")

    if spells == "*":
        if params["state"] == "latest":
            # back up original queue
            try:
                os.rename(sorcery_queue, f"{sorcery_queue}.backup")
            except OSError:
                module.fail_json(msg="failed to backup the update queue")

            # see update_codex()
            module.run_command_environ_update.update(dict(SILENT="1"))

            cmd_sorcery = f"{SORCERY['sorcery']} queue"

            rc, stdout, stderr = module.run_command(cmd_sorcery)

            if rc != 0:
                module.fail_json(msg="failed to generate the update queue")

            try:
                queue_size = os.stat(sorcery_queue).st_size
            except Exception:
                module.fail_json(msg="failed to read the update queue")

            if queue_size != 0:
                if module.check_mode:
                    try:
                        os.rename(f"{sorcery_queue}.backup", sorcery_queue)
                    except OSError:
                        module.fail_json(msg="failed to restore the update queue")

                    return (True, "would have updated the system")

                cmd_cast = f"{SORCERY['cast']} --queue"

                rc, stdout, stderr = module.run_command(cmd_cast)

                if rc != 0:
                    module.fail_json(msg="failed to update the system")

                return (True, "successfully updated the system")
            else:
                return (False, "the system is already up to date")
        elif params["state"] == "rebuild":
            if module.check_mode:
                return (True, "would have rebuilt the system")

            cmd_sorcery = f"{SORCERY['sorcery']} rebuild"

            rc, stdout, stderr = module.run_command(cmd_sorcery)

            if rc != 0:
                module.fail_json(msg=f"failed to rebuild the system: {stdout}")

            return (True, "successfully rebuilt the system")
        else:
            module.fail_json(msg="unsupported operation on '*' name value")
    else:
        if params["state"] in ("present", "latest", "rebuild", "absent"):
            # extract versions from the 'gaze' command
            cmd_gaze = f"{SORCERY['gaze']} -q version {' '.join(spells)}"

            rc, stdout, stderr = module.run_command(cmd_gaze)

            # fail if any of spells cannot be found
            if rc != 0:
                module.fail_json(msg=f"failed to locate spell(s) in the list ({', '.join(spells)})")

            cast_queue = []
            dispel_queue = []

            rex = re.compile(r"[^|]+\|[^|]+\|(?P<spell>[^|]+)\|(?P<grim_ver>[^|]+)\|(?P<inst_ver>[^$]+)")

            # drop 2-line header and empty trailing line
            for line in stdout.splitlines()[2:-1]:
                match = rex.match(line)

                cast = False

                if params["state"] == "present":
                    # spell is not installed..
                    if match.group("inst_ver") == "-":
                        # ..so set up depends reqs for it
                        match_depends(module)

                        cast = True
                    # spell is installed..
                    else:
                        # ..but does not conform depends reqs
                        if not match_depends(module):
                            cast = True
                elif params["state"] == "latest":
                    # grimoire and installed versions do not match..
                    if match.group("grim_ver") != match.group("inst_ver"):
                        # ..so check for depends reqs first and set them up
                        match_depends(module)

                        cast = True
                    # grimoire and installed versions match..
                    else:
                        # ..but the spell does not conform depends reqs
                        if not match_depends(module):
                            cast = True
                elif params["state"] == "rebuild":
                    cast = True
                # 'absent'
                else:
                    if match.group("inst_ver") != "-":
                        dispel_queue.append(match.group("spell"))

                if cast:
                    cast_queue.append(match.group("spell"))

            if cast_queue:
                if module.check_mode:
                    return (True, "would have cast spell(s)")

                cmd_cast = f"{SORCERY['cast']} -c {' '.join(cast_queue)}"

                rc, stdout, stderr = module.run_command(cmd_cast)

                if rc != 0:
                    module.fail_json(msg=f"failed to cast spell(s): {stdout}")

                return (True, "successfully cast spell(s)")
            elif params["state"] != "absent":
                return (False, "spell(s) are already cast")

            if dispel_queue:
                if module.check_mode:
                    return (True, "would have dispelled spell(s)")

                cmd_dispel = f"{SORCERY['dispel']} {' '.join(dispel_queue)}"

                rc, stdout, stderr = module.run_command(cmd_dispel)

                if rc != 0:
                    module.fail_json(msg=f"failed to dispel spell(s): {stdout}")

                return (True, "successfully dispelled spell(s)")
            else:
                return (False, "spell(s) are already dispelled")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(aliases=["spell", "grimoire"], type="list", elements="str"),
            repository=dict(type="str"),
            state=dict(default="present", choices=["present", "latest", "absent", "cast", "dispelled", "rebuild"]),
            depends=dict(),
            update=dict(default=False, type="bool"),
            update_cache=dict(default=False, aliases=["update_codex"], type="bool"),
            cache_valid_time=dict(default=0, type="int"),
        ),
        required_one_of=[["name", "update", "update_cache"]],
        supports_check_mode=True,
    )

    if os.geteuid() != 0:
        module.fail_json(msg="root privileges are required for this operation")

    for c in SORCERY:
        SORCERY[c] = module.get_bin_path(c, True)

    # prepare environment: run sorcery commands without asking questions
    module.run_command_environ_update = dict(PROMPT_DELAY="0", VOYEUR="0", LANGUAGE="C", LC_ALL="C")

    params = module.params

    # normalize 'state' parameter
    if params["state"] in ("present", "cast"):
        params["state"] = "present"
    elif params["state"] in ("absent", "dispelled"):
        params["state"] = "absent"

    changed = {"sorcery": (False, NA), "grimoires": (False, NA), "codex": (False, NA), "spells": (False, NA)}

    if params["update"]:
        changed["sorcery"] = update_sorcery(module)

    if params["name"] and params["repository"]:
        changed["grimoires"] = manage_grimoires(module)

    if params["update_cache"]:
        changed["codex"] = update_codex(module)

    if params["name"] and not params["repository"]:
        changed["spells"] = manage_spells(module)

    if any(x[0] for x in changed.values()):
        state_msg = "state changed"
        state_changed = True
    else:
        state_msg = "no change in state"
        state_changed = False

    module.exit_json(changed=state_changed, msg=f"{state_msg}: {'; '.join(x[1] for x in changed.values())}")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016-2017, Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_logical_interconnect_group_info
short_description: Retrieve information about one or more of the OneView Logical Interconnect Groups
description:
  - Retrieve information about one or more of the Logical Interconnect Groups from OneView.
requirements:
  - hpOneView >= 2.0.1
author:
  - Felipe Bulsoni (@fgbulsoni)
  - Thiago Miotto (@tmiotto)
  - Adriane Cardozo (@adriane-cardozo)
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  name:
    description:
      - Logical Interconnect Group name.
    type: str
extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.factsparams
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather information about all Logical Interconnect Groups
  community.general.oneview_logical_interconnect_group_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Logical Interconnect Groups
  ansible.builtin.debug:
    msg: "{{ result.logical_interconnect_groups }}"

- name: Gather paginated, filtered and sorted information about Logical Interconnect Groups
  community.general.oneview_logical_interconnect_group_info:
    params:
      start: 0
      count: 3
      sort: name:descending
      filter: name=LIGName
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about paginated, filtered and sorted list of Logical Interconnect Groups
  ansible.builtin.debug:
    msg: "{{ result.logical_interconnect_groups }}"

- name: Gather information about a Logical Interconnect Group by name
  community.general.oneview_logical_interconnect_group_info:
    name: logical interconnect group name
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Logical Interconnect Group found by name
  ansible.builtin.debug:
    msg: "{{ result.logical_interconnect_groups }}"
"""

RETURN = r"""
logical_interconnect_groups:
  description: Has all the OneView information about the Logical Interconnect Groups.
  returned: Always, but can be null.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import OneViewModuleBase


class LogicalInterconnectGroupInfoModule(OneViewModuleBase):
    def __init__(self):
        argument_spec = dict(
            name=dict(type="str"),
            params=dict(type="dict"),
        )

        super().__init__(
            additional_arg_spec=argument_spec,
            supports_check_mode=True,
        )

    def execute_module(self):
        if self.module.params.get("name"):
            ligs = self.oneview_client.logical_interconnect_groups.get_by("name", self.module.params["name"])
        else:
            ligs = self.oneview_client.logical_interconnect_groups.get_all(**self.facts_params)

        return dict(changed=False, logical_interconnect_groups=ligs)


def main():
    LogicalInterconnectGroupInfoModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019 David Lundgren <dlundgren@syberisle.net>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - David Lundgren (@dlundgren)
module: sysrc
short_description: Manage FreeBSD using sysrc
version_added: '2.0.0'
description:
  - Manages C(/etc/rc.conf) for FreeBSD.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of variable in C(/etc/rc.conf) to manage.
    type: str
    required: true
  value:
    description:
      - The value to set when O(state=present).
      - The value to add when O(state=value_present).
      - The value to remove when O(state=value_absent).
    type: str
  state:
    description:
      - Use V(present) to add the variable.
      - Use V(absent) to remove the variable.
      - Use V(value_present) to add the value to the existing variable.
      - Use V(value_absent) to remove the value from the existing variable.
    type: str
    default: "present"
    choices: [absent, present, value_present, value_absent]
  path:
    description:
      - Path to file to use instead of V(/etc/rc.conf).
    type: str
    default: "/etc/rc.conf"
  delim:
    description:
      - Delimiter to be used instead of V(" ") (space).
      - Only used when O(state=value_present) or O(state=value_absent).
    default: " "
    type: str
  jail:
    description:
      - Name or ID of the jail to operate on.
    type: str
notes:
  - The O(name) cannot contain periods as sysrc does not support OID style names.
"""

EXAMPLES = r"""
# enable mysql in the /etc/rc.conf
- name: Configure mysql pid file
  community.general.sysrc:
    name: mysql_pidfile
    value: "/var/run/mysqld/mysqld.pid"

# enable accf_http kld in the boot loader
- name: Enable accf_http kld
  community.general.sysrc:
    name: accf_http_load
    state: present
    value: "YES"
    path: /boot/loader.conf

# add gif0 to cloned_interfaces
- name: Add gif0 interface
  community.general.sysrc:
    name: cloned_interfaces
    state: value_present
    value: "gif0"

# enable nginx on a jail
- name: Enable nginx in test jail
  community.general.sysrc:
    name: nginx_enable
    value: "YES"
    jail: testjail
"""


import os
import re

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper


class Sysrc(StateModuleHelper):
    module = dict(
        argument_spec=dict(
            name=dict(type="str", required=True),
            value=dict(type="str"),
            state=dict(type="str", default="present", choices=["absent", "present", "value_present", "value_absent"]),
            path=dict(type="str", default="/etc/rc.conf"),
            delim=dict(type="str", default=" "),
            jail=dict(type="str"),
        ),
        supports_check_mode=True,
    )
    output_params = ("value",)
    use_old_vardict = False

    def __init_module__(self):
        # OID style names are not supported
        if not re.match(r"^\w+$", self.vars.name, re.ASCII):
            self.module.fail_json(msg="Name may only contain alpha-numeric and underscore characters")

        self.module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
        self.sysrc = self.module.get_bin_path("sysrc", True)

    def _contains(self):
        value = self._get()
        if value is None:
            return False, None

        value = value.split(self.vars.delim)

        return self.vars.value in value, value

    def _get(self):
        if not os.path.exists(self.vars.path):
            return None

        (rc, out, err) = self._sysrc("-v", "-n", self.vars.name)
        if "unknown variable" in err or "unknown variable" in out:
            # Prior to FreeBSD 11.1 sysrc would write "unknown variable" to stdout and not stderr
            # https://bugs.freebsd.org/bugzilla/show_bug.cgi?id=229806
            return None

        if out.startswith(self.vars.path):
            return out.split(":", 1)[1].strip()

        return None

    def _modify(self, op, changed):
        (rc, out, err) = self._sysrc(f"{self.vars.name}{op}={self.vars.delim}{self.vars.value}")
        if out.startswith(f"{self.vars.name}:"):
            return changed(out.split(" -> ")[1].strip().split(self.vars.delim))

        return False

    def _sysrc(self, *args):
        cmd = [self.sysrc, "-f", self.vars.path]
        if self.vars.jail:
            cmd += ["-j", self.vars.jail]
        cmd.extend(args)

        (rc, out, err) = self.module.run_command(cmd)
        if "Permission denied" in err:
            self.module.fail_json(msg=f"Permission denied for {self.vars.path}")

        return rc, out, err

    def state_absent(self):
        if self._get() is None:
            return

        if not self.check_mode:
            self._sysrc("-x", self.vars.name)

        self.changed = True

    def state_present(self):
        value = self._get()
        if value == self.vars.value:
            return

        if self.vars.value is None:
            self.vars.set("value", value)
            return

        if not self.check_mode:
            self._sysrc(f"{self.vars.name}={self.vars.value}")

        self.changed = True

    def state_value_absent(self):
        (contains, _unused) = self._contains()
        if not contains:
            return

        self.changed = self.check_mode or self._modify("-", lambda values: self.vars.value not in values)

    def state_value_present(self):
        (contains, value) = self._contains()
        if contains:
            return

        if self.vars.value is None:
            self.vars.set("value", value)
            return

        self.changed = self.check_mode or self._modify("+", lambda values: self.vars.value in values)


def main():
    Sysrc.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: python_requirements_info
short_description: Show python path and assert dependency versions
description:
  - Get info about available Python requirements on the target host, including listing required libraries and gathering versions.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  dependencies:
    type: list
    elements: str
    description:
      - 'A list of version-likes or module names to check for installation. Supported operators: C(<), C(>), C(<=), C(>=),
        or C(==).'
      - The bare module name like V(ansible), the module with a specific version like V(boto3==1.6.1), or a partial version
        like V(requests>2) are all valid specifications.
    default: []
author:
  - Will Thames (@willthames)
  - Ryan Scott Brown (@ryansb)
"""

EXAMPLES = r"""
- name: Show python lib/site paths
  community.general.python_requirements_info:

- name: Check for modern boto3 and botocore versions
  community.general.python_requirements_info:
    dependencies:
      - boto3>1.6
      - botocore<2
"""

RETURN = r"""
python:
  description: Path to the Python interpreter used.
  returned: always
  type: str
  sample: /usr/local/opt/python@2/bin/python2.7
python_version:
  description: Version of Python.
  returned: always
  type: str
  sample: "2.7.15 (default, May  1 2018, 16:44:08)\n[GCC 4.2.1 Compatible Apple LLVM 9.1.0 (clang-902.0.39.1)]"
python_version_info:
  description: Breakdown version of Python.
  returned: always
  type: dict
  contains:
    major:
      description: The C(major) component of the python interpreter version.
      returned: always
      type: int
      sample: 3
    minor:
      description: The C(minor) component of the python interpreter version.
      returned: always
      type: int
      sample: 8
    micro:
      description: The C(micro) component of the python interpreter version.
      returned: always
      type: int
      sample: 10
    releaselevel:
      description: The C(releaselevel) component of the python interpreter version.
      returned: always
      type: str
      sample: final
    serial:
      description: The C(serial) component of the python interpreter version.
      returned: always
      type: int
      sample: 0
  version_added: 4.2.0
python_system_path:
  description: List of paths Python is looking for modules in.
  returned: always
  type: list
  sample:
    - /usr/local/opt/python@2/site-packages/
    - /usr/lib/python/site-packages/
valid:
  description: A dictionary of dependencies that matched their desired versions. If no version was specified, then RV(ignore:desired)
    is V(null).
  returned: always
  type: dict
  sample:
    boto3:
      desired:
      installed: 1.7.60
    botocore:
      desired: botocore<2
      installed: 1.10.60
mismatched:
  description: A dictionary of dependencies that did not satisfy the desired version.
  returned: always
  type: dict
  sample:
    botocore:
      desired: botocore>2
      installed: 1.10.60
not_found:
  description: A list of packages that could not be imported at all, and are not installed.
  returned: always
  type: list
  sample:
    - boto4
    - requests
"""

import operator
import re
import sys

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

HAS_IMPORTLIB_METADATA = False
try:
    import importlib.metadata

    HAS_IMPORTLIB_METADATA = True
except ImportError:
    pass

HAS_DISTUTILS = False
try:
    import pkg_resources

    HAS_DISTUTILS = True
except ImportError:
    pass

operations = {
    "<=": operator.le,
    ">=": operator.ge,
    "<": operator.lt,
    ">": operator.gt,
    "==": operator.eq,
}

python_version_info = dict(
    major=sys.version_info[0],
    minor=sys.version_info[1],
    micro=sys.version_info[2],
    releaselevel=sys.version_info[3],
    serial=sys.version_info[4],
)


def main():
    module = AnsibleModule(
        argument_spec=dict(dependencies=dict(type="list", elements="str", default=[])),
        supports_check_mode=True,
    )
    if not HAS_DISTUTILS and not HAS_IMPORTLIB_METADATA:
        module.fail_json(
            msg='Could not import "pkg_resources" or "importlib.metadata" libraries to introspect Python environment.',
            python=sys.executable,
            python_version=sys.version,
            python_version_info=python_version_info,
            python_system_path=sys.path,
        )
    pkg_dep_re = re.compile(r"(^[a-zA-Z][a-zA-Z0-9_-]+)(?:(==|[><]=?)([0-9.]+))?$")

    results = dict(
        not_found=[],
        mismatched={},
        valid={},
    )

    for dep in module.params["dependencies"]:
        match = pkg_dep_re.match(dep)
        if not match:
            module.fail_json(msg=f"Failed to parse version requirement '{dep}'. Must be formatted like 'ansible>2.6'")
        pkg, op, version = match.groups()
        if op is not None and op not in operations:
            module.fail_json(
                msg=f"Failed to parse version requirement '{dep}'. Operator must be one of >, <, <=, >=, or =="
            )
        if HAS_DISTUTILS:
            try:
                existing = pkg_resources.get_distribution(pkg).version
            except pkg_resources.DistributionNotFound:
                # not there
                results["not_found"].append(pkg)
                continue
        else:
            try:
                existing = importlib.metadata.version(pkg)
            except importlib.metadata.PackageNotFoundError:
                # not there
                results["not_found"].append(pkg)
                continue
        if op is None and version is None:
            results["valid"][pkg] = {
                "installed": existing,
                "desired": None,
            }
        elif operations[op](LooseVersion(existing), LooseVersion(version)):
            results["valid"][pkg] = {
                "installed": existing,
                "desired": dep,
            }
        else:
            results["mismatched"][pkg] = {
                "installed": existing,
                "desired": dep,
            }

    module.exit_json(
        python=sys.executable,
        python_version=sys.version,
        python_version_info=python_version_info,
        python_system_path=sys.path,
        **results,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2017, Loic Blot <loic.blot@unix-experience.fr>
# Sponsored by Infopro Digital. http://www.infopro-digital.com/
# Sponsored by E.T.A.I. http://www.etai.fr/
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rundeck_acl_policy

short_description: Manage Rundeck ACL policies
description:
  - Create, update and remove Rundeck ACL policies through HTTP API.
author: "Loic Blot (@nerzhul)"
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    type: str
    description:
      - Create or remove Rundeck project.
    choices: ['present', 'absent']
    default: 'present'
  name:
    type: str
    description:
      - Sets the project name.
    required: true
  api_token:
    description:
      - Sets the token to authenticate against Rundeck API.
    aliases: ["token"]
  project:
    type: str
    description:
      - Sets the project which receive the ACL policy.
      - If unset, it is a system ACL policy.
  policy:
    type: str
    description:
      - Sets the ACL policy content.
      - ACL policy content is a YAML object as described in U(http://rundeck.org/docs/man5/aclpolicy.html).
      - It can be a YAML string or a pure Ansible inventory YAML object.
  client_cert:
    version_added: '0.2.0'
  client_key:
    version_added: '0.2.0'
  force:
    version_added: '0.2.0'
  force_basic_auth:
    version_added: '0.2.0'
  http_agent:
    version_added: '0.2.0'
  url_password:
    version_added: '0.2.0'
  url_username:
    version_added: '0.2.0'
  use_proxy:
    version_added: '0.2.0'
  validate_certs:
    version_added: '0.2.0'
extends_documentation_fragment:
  - ansible.builtin.url
  - community.general._attributes
  - community.general._rundeck
"""

EXAMPLES = r"""
- name: Create or update a rundeck ACL policy in project Ansible
  community.general.rundeck_acl_policy:
    name: "Project_01"
    api_version: 18
    url: "https://rundeck.example.org"
    token: "mytoken"
    state: present
    project: "Ansible"
    policy:
      description: "my policy"
      context:
        application: rundeck
      for:
        project:
          - allow: read
      by:
        group: "build"

- name: Remove a rundeck system policy
  community.general.rundeck_acl_policy:
    name: "Project_01"
    url: "https://rundeck.example.org"
    token: "mytoken"
    state: absent
"""

RETURN = r"""
rundeck_response:
  description: Rundeck response when a failure occurs.
  returned: failed
  type: str
before:
  description: Dictionary containing ACL policy information before modification.
  returned: success
  type: dict
after:
  description: Dictionary containing ACL policy information after modification.
  returned: success
  type: dict
"""

# import module snippets
import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._rundeck import (
    api_argument_spec,
    api_request,
)


class RundeckACLManager:
    def __init__(self, module):
        self.module = module
        if module.params.get("project"):
            self.endpoint = f"project/{self.module.params['project']}/acl/{self.module.params['name']}.aclpolicy"
        else:
            self.endpoint = f"system/acl/{self.module.params['name']}.aclpolicy"

    def get_acl(self):
        resp, info = api_request(
            module=self.module,
            endpoint=self.endpoint,
        )

        return resp

    def create_or_update_acl(self):
        facts = self.get_acl()
        if facts is None:
            # If in check mode don't create project, simulate a fake project creation
            if self.module.check_mode:
                self.module.exit_json(changed=True, before={}, after=self.module.params["policy"])

            resp, info = api_request(
                module=self.module,
                endpoint=self.endpoint,
                method="POST",
                data={"contents": self.module.params["policy"]},
            )

            if info["status"] == 201:
                self.module.exit_json(changed=True, before={}, after=self.get_acl())
            elif info["status"] == 400:
                self.module.fail_json(
                    msg=f"Unable to validate acl {self.module.params['name']}. Please ensure it is a valid ACL"
                )
            elif info["status"] == 409:
                self.module.fail_json(msg=f"ACL {self.module.params['name']} already exists")
            else:
                self.module.fail_json(
                    msg=f"Unhandled HTTP status {info['status']}, please report the bug",
                    before={},
                    after=self.get_acl(),
                )
        else:
            if facts["contents"] == self.module.params["policy"]:
                self.module.exit_json(changed=False, before=facts, after=facts)

            if self.module.check_mode:
                self.module.exit_json(changed=True, before=facts, after=facts)

            resp, info = api_request(
                module=self.module,
                endpoint=self.endpoint,
                method="PUT",
                data={"contents": self.module.params["policy"]},
            )

            if info["status"] == 200:
                self.module.exit_json(changed=True, before=facts, after=self.get_acl())
            elif info["status"] == 400:
                self.module.fail_json(
                    msg=f"Unable to validate acl {self.module.params['name']}. Please ensure it is a valid ACL"
                )
            elif info["status"] == 404:
                self.module.fail_json(msg=f"ACL {self.module.params['name']} doesn't exists. Cannot update.")

    def remove_acl(self):
        facts = self.get_acl()

        if facts is None:
            self.module.exit_json(changed=False, before={}, after={})
        else:
            # If not in check mode, remove the project
            if not self.module.check_mode:
                api_request(
                    module=self.module,
                    endpoint=self.endpoint,
                    method="DELETE",
                )

                self.module.exit_json(changed=True, before=facts, after={})


def main():
    # Also allow the user to set values for fetch_url
    argument_spec = api_argument_spec()
    argument_spec.update(
        dict(
            state=dict(type="str", choices=["present", "absent"], default="present"),
            name=dict(required=True, type="str"),
            policy=dict(type="str"),
            project=dict(type="str"),
        )
    )

    argument_spec["api_token"]["aliases"] = ["token"]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=[
            ["state", "present", ["policy"]],
        ],
        supports_check_mode=True,
    )

    if not bool(re.match("[a-zA-Z0-9,.+_-]+", module.params["name"])):
        module.fail_json(
            msg="Name contains forbidden characters. The policy can contain the characters: a-zA-Z0-9,.+_-"
        )

    if module.params["api_version"] < 14:
        module.fail_json(msg="API version should be at least 14")

    rundeck = RundeckACLManager(module)
    if module.params["state"] == "present":
        rundeck.create_or_update_acl()
    elif module.params["state"] == "absent":
        rundeck.remove_acl()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Nurfet Becirevic <nurfet.becirevic@gmail.com>
# Copyright (c) 2017, Tomas Karasek <tom.to.the.k@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: packet_ip_subnet

short_description: Assign IP subnet to a bare metal server

description:
  - Assign or unassign IPv4 or IPv6 subnets to or from a device in the Packet host.
  - IPv4 subnets must come from already reserved block.
  - IPv6 subnets must come from publicly routable /56 block from your project.
  - See U(https://support.packet.com/kb/articles/elastic-ips) for more info on IP block reservation.
version_added: '0.2.0'

author:
  - Tomas Karasek (@t0mk) <tom.to.the.k@gmail.com>
  - Nurfet Becirevic (@nurfet-becirevic) <nurfet.becirevic@gmail.com>

extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  auth_token:
    description:
      - Packet API token. You can also supply it in environment variable E(PACKET_API_TOKEN).
    type: str

  hostname:
    description:
      - A hostname of a device to/from which to assign/remove a subnet.
    type: str

  device_id:
    description:
      - UUID of a device to/from which to assign/remove a subnet.
    type: str

  project_id:
    description:
      - UUID of a project of the device to/from which to assign/remove a subnet.
    type: str

  device_count:
    description:
      - The number of devices to retrieve from the project. The max allowed value is 1000.
      - See U(https://www.packet.com/developers/api/#retrieve-all-devices-of-a-project) for more info.
    default: 100
    type: int

  cidr:
    description:
      - IPv4 or IPv6 subnet which you want to manage. It must come from a reserved block for your project in the Packet Host.
    aliases: [name]
    type: str
    required: true

  state:
    description:
      - Desired state of the IP subnet on the specified device.
      - With O(state=present), you must specify either O(hostname) or O(device_id). Subnet with given CIDR is then assigned
        to the specified device.
      - With O(state=absent), you can specify either O(hostname) or O(device_id). The subnet is then removed from specified
        devices.
      - If you leave both O(hostname) and O(device_id) empty, the subnet is then removed from any device it is assigned to.
    choices: ['present', 'absent']
    default: 'present'
    type: str

requirements:
  - "packet-python >= 1.35"
"""

EXAMPLES = r"""
# All the examples assume that you have your Packet API token in env var PACKET_API_TOKEN.
# You can also pass it to the auth_token parameter of the module instead.

- name: Create 1 device and assign an arbitrary public IPv4 subnet to it
  hosts: localhost
  tasks:

    - packet_device:
        project_id: 89b497ee-5afc-420a-8fb5-56984898f4df
        hostnames: myserver
        operating_system: ubuntu_16_04
        plan: baremetal_0
        facility: sjc1
        state: active

# Pick an IPv4 address from a block allocated to your project.

    - community.general.packet_ip_subnet:
        project_id: 89b497ee-5afc-420a-8fb5-56984898f4df
        hostname: myserver
        cidr: "147.75.201.78/32"

# Release IP address 147.75.201.78

- name: Unassign IP address from any device in your project
  hosts: localhost
  tasks:
    - community.general.packet_ip_subnet:
        project_id: 89b497ee-5afc-420a-8fb5-56984898f4df
        cidr: "147.75.201.78/32"
        state: absent
"""

RETURN = r"""
device_id:
  type: str
  description: UUID of the device associated with the specified IP address.
  returned: success

subnet:
  description: Dict with data about the handled IP subnet.
  type: dict
  sample:
    address: 147.75.90.241
    address_family: 4
    assigned_to: {href: /devices/61f9aa5e-0530-47f5-97c2-113828e61ed0}
    cidr: 31
    created_at: '2017-08-07T15:15:30Z'
    enabled: true
    gateway: 147.75.90.240
    href: /ips/31eda960-0a16-4c0f-b196-f3dc4928529f
    id: 1eda960-0a16-4c0f-b196-f3dc4928529f
    manageable: true
    management: true
    netmask: 255.255.255.254
    network: 147.75.90.240
    public: true
  returned: success
"""


import re
import uuid

from ansible.module_utils.basic import AnsibleModule, env_fallback

HAS_PACKET_SDK = True

try:
    import packet
except ImportError:
    HAS_PACKET_SDK = False


NAME_RE = r"([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])"
HOSTNAME_RE = rf"({NAME_RE}\.)*{NAME_RE}$"
PROJECT_MAX_DEVICES = 100


PACKET_API_TOKEN_ENV_VAR = "PACKET_API_TOKEN"


ALLOWED_STATES = ["absent", "present"]


def is_valid_hostname(hostname):
    return re.match(HOSTNAME_RE, hostname) is not None


def is_valid_uuid(myuuid):
    try:
        val = uuid.UUID(myuuid, version=4)
    except ValueError:
        return False
    return str(val) == myuuid


def get_existing_devices(module, packet_conn):
    project_id = module.params.get("project_id")
    if not is_valid_uuid(project_id):
        raise Exception(f"Project ID {project_id} does not seem to be valid")

    per_page = module.params.get("device_count")
    return packet_conn.list_devices(project_id, params={"per_page": per_page})


def get_specified_device_identifiers(module):
    if module.params.get("device_id"):
        _d_id = module.params.get("device_id")
        if not is_valid_uuid(_d_id):
            raise Exception(f"Device ID '{_d_id}' does not seem to be valid")
        return {"device_id": _d_id, "hostname": None}
    elif module.params.get("hostname"):
        _hn = module.params.get("hostname")
        if not is_valid_hostname(_hn):
            raise Exception(f"Hostname '{_hn}' does not seem to be valid")
        return {"hostname": _hn, "device_id": None}
    else:
        return {"hostname": None, "device_id": None}


def parse_subnet_cidr(cidr):
    if "/" not in cidr:
        raise Exception("CIDR expression in wrong format, must be address/prefix_len")
    addr, prefixlen = cidr.split("/")
    try:
        prefixlen = int(prefixlen)
    except ValueError as e:
        raise Exception(f"Wrong prefix length in CIDR expression {cidr}") from e
    return addr, prefixlen


def act_on_assignment(target_state, module, packet_conn):
    return_dict = {"changed": False}
    specified_cidr = module.params.get("cidr")
    address, prefixlen = parse_subnet_cidr(specified_cidr)

    specified_identifier = get_specified_device_identifiers(module)

    if module.check_mode:
        return return_dict

    if (specified_identifier["hostname"] is None) and (specified_identifier["device_id"] is None):
        if target_state == "absent":
            # The special case to release the IP from any assignment
            for d in get_existing_devices(module, packet_conn):
                for ia in d.ip_addresses:
                    if address == ia["address"] and prefixlen == ia["cidr"]:
                        packet_conn.call_api(ia["href"], "DELETE")
                        return_dict["changed"] = True
                        return_dict["subnet"] = ia
                        return_dict["device_id"] = d.id
                        return return_dict
        raise Exception("If you assign an address, you must specify either target device ID or target unique hostname.")

    if specified_identifier["device_id"] is not None:
        device = packet_conn.get_device(specified_identifier["device_id"])
    else:
        all_devices = get_existing_devices(module, packet_conn)
        hn = specified_identifier["hostname"]
        matching_devices = [d for d in all_devices if d.hostname == hn]
        if len(matching_devices) > 1:
            raise Exception(f"There are more than one devices matching given hostname {hn}")
        if len(matching_devices) == 0:
            raise Exception(f"There is no device matching given hostname {hn}")
        device = matching_devices[0]

    return_dict["device_id"] = device.id
    assignment_dicts = [i for i in device.ip_addresses if i["address"] == address and i["cidr"] == prefixlen]
    if len(assignment_dicts) > 1:
        raise Exception(f"IP address {specified_cidr} is assigned more than once for device {device.hostname}")

    if target_state == "absent":
        if len(assignment_dicts) == 1:
            packet_conn.call_api(assignment_dicts[0]["href"], "DELETE")
            return_dict["subnet"] = assignment_dicts[0]
            return_dict["changed"] = True
    elif target_state == "present":
        if len(assignment_dicts) == 0:
            new_assignment = packet_conn.call_api(f"devices/{device.id}/ips", "POST", {"address": f"{specified_cidr}"})
            return_dict["changed"] = True
            return_dict["subnet"] = new_assignment
    return return_dict


def main():
    module = AnsibleModule(
        argument_spec=dict(
            auth_token=dict(type="str", fallback=(env_fallback, [PACKET_API_TOKEN_ENV_VAR]), no_log=True),
            device_id=dict(type="str"),
            hostname=dict(type="str"),
            project_id=dict(type="str"),
            device_count=dict(type="int", default=PROJECT_MAX_DEVICES),
            cidr=dict(type="str", required=True, aliases=["name"]),
            state=dict(choices=ALLOWED_STATES, default="present"),
        ),
        supports_check_mode=True,
        mutually_exclusive=[("hostname", "device_id")],
        required_one_of=[["hostname", "device_id", "project_id"]],
        required_by=dict(
            hostname=("project_id",),
        ),
    )

    if not HAS_PACKET_SDK:
        module.fail_json(msg="packet required for this module")

    if not module.params.get("auth_token"):
        _fail_msg = f"if Packet API token is not in environment variable {PACKET_API_TOKEN_ENV_VAR}, the auth_token parameter is required"
        module.fail_json(msg=_fail_msg)

    auth_token = module.params.get("auth_token")

    packet_conn = packet.Manager(auth_token=auth_token)

    state = module.params.get("state")

    try:
        module.exit_json(**act_on_assignment(state, module, packet_conn))
    except Exception as e:
        module.fail_json(msg=f"failed to set IP subnet to state {state}, error: {e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Petr Lautrbach <plautrba@redhat.com>
# Based on seport.py module (c) 2014, Dan Keder <dan.keder@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: selogin
short_description: Manages linux user to SELinux user mapping
description:
  - Manages linux user to SELinux user mapping.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  login:
    type: str
    description:
      - A Linux user.
    required: true
  seuser:
    type: str
    description:
      - SELinux user name.
  selevel:
    type: str
    aliases: [serange]
    description:
      - MLS/MCS Security Range (MLS/MCS Systems only) SELinux Range for SELinux login mapping defaults to the SELinux user
        record range.
    default: s0
  state:
    type: str
    description:
      - Desired mapping value.
    default: present
    choices: ['present', 'absent']
  reload:
    description:
      - Reload SELinux policy after commit.
    type: bool
    default: true
  ignore_selinux_state:
    description:
      - Run independent of selinux runtime state.
    type: bool
    default: false
notes:
  - The changes are persistent across reboots.
  - Not tested on any debian based system.
requirements: ['libselinux', 'policycoreutils']
author:
  - Dan Keder (@dankeder)
  - Petr Lautrbach (@bachradsusi)
  - James Cassell (@jamescassell)
"""

EXAMPLES = r"""
- name: Modify the default user on the system to the guest_u user
  community.general.selogin:
    login: __default__
    seuser: guest_u
    state: present

- name: Assign gijoe user on an MLS machine a range and to the staff_u user
  community.general.selogin:
    login: gijoe
    seuser: staff_u
    serange: SystemLow-Secret
    state: present

- name: Assign all users in the engineering group to the staff_u user
  community.general.selogin:
    login: '%engineering'
    seuser: staff_u
    state: present
"""

RETURN = r"""
# Default return values
"""


import traceback

SELINUX_IMP_ERR = None
try:
    import selinux

    HAVE_SELINUX = True
except ImportError:
    SELINUX_IMP_ERR = traceback.format_exc()
    HAVE_SELINUX = False

SEOBJECT_IMP_ERR = None
try:
    import seobject

    HAVE_SEOBJECT = True
except ImportError:
    SEOBJECT_IMP_ERR = traceback.format_exc()
    HAVE_SEOBJECT = False


from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def semanage_login_add(module, login, seuser, do_reload, serange="s0", sestore=""):
    """Add linux user to SELinux user mapping

    :type module: AnsibleModule
    :param module: Ansible module

    :type login: str
    :param login: a Linux User or a Linux group if it begins with %

    :type seuser: str
    :param proto: An SELinux user ('__default__', 'unconfined_u', 'staff_u', ...), see 'semanage login -l'

    :type serange: str
    :param serange: SELinux MLS/MCS range (defaults to 's0')

    :type do_reload: bool
    :param do_reload: Whether to reload SELinux policy after commit

    :type sestore: str
    :param sestore: SELinux store

    :rtype: bool
    :return: True if the policy was changed, otherwise False
    """
    try:
        selogin = seobject.loginRecords(sestore)
        selogin.set_reload(do_reload)
        change = False
        all_logins = selogin.get_all()
        # module.fail_json(msg="%s: %s %s" % (all_logins, login, sestore))
        # for local_login in all_logins:
        if login not in all_logins.keys():
            change = True
            if not module.check_mode:
                selogin.add(login, seuser, serange)
        else:
            if all_logins[login][0] != seuser or all_logins[login][1] != serange:
                change = True
                if not module.check_mode:
                    selogin.modify(login, seuser, serange)

    except (ValueError, KeyError, OSError, RuntimeError) as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n", exception=traceback.format_exc())

    return change


def semanage_login_del(module, login, seuser, do_reload, sestore=""):
    """Delete linux user to SELinux user mapping

    :type module: AnsibleModule
    :param module: Ansible module

    :type login: str
    :param login: a Linux User or a Linux group if it begins with %

    :type seuser: str
    :param proto: An SELinux user ('__default__', 'unconfined_u', 'staff_u', ...), see 'semanage login -l'

    :type do_reload: bool
    :param do_reload: Whether to reload SELinux policy after commit

    :type sestore: str
    :param sestore: SELinux store

    :rtype: bool
    :return: True if the policy was changed, otherwise False
    """
    try:
        selogin = seobject.loginRecords(sestore)
        selogin.set_reload(do_reload)
        change = False
        all_logins = selogin.get_all()
        # module.fail_json(msg="%s: %s %s" % (all_logins, login, sestore))
        if login in all_logins.keys():
            change = True
            if not module.check_mode:
                selogin.delete(login)

    except (ValueError, KeyError, OSError, RuntimeError) as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n", exception=traceback.format_exc())

    return change


def get_runtime_status(ignore_selinux_state=False):
    return True if ignore_selinux_state is True else selinux.is_selinux_enabled()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            ignore_selinux_state=dict(type="bool", default=False),
            login=dict(type="str", required=True),
            seuser=dict(type="str"),
            selevel=dict(type="str", aliases=["serange"], default="s0"),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            reload=dict(type="bool", default=True),
        ),
        required_if=[["state", "present", ["seuser"]]],
        supports_check_mode=True,
    )
    if not HAVE_SELINUX:
        module.fail_json(msg=missing_required_lib("libselinux"), exception=SELINUX_IMP_ERR)

    if not HAVE_SEOBJECT:
        module.fail_json(msg=missing_required_lib("seobject from policycoreutils"), exception=SEOBJECT_IMP_ERR)

    ignore_selinux_state = module.params["ignore_selinux_state"]

    if not get_runtime_status(ignore_selinux_state):
        module.fail_json(msg="SELinux is disabled on this host.")

    login = module.params["login"]
    seuser = module.params["seuser"]
    serange = module.params["selevel"]
    state = module.params["state"]
    do_reload = module.params["reload"]

    result = {
        "login": login,
        "seuser": seuser,
        "serange": serange,
        "state": state,
    }

    if state == "present":
        result["changed"] = semanage_login_add(module, login, seuser, do_reload, serange)
    elif state == "absent":
        result["changed"] = semanage_login_del(module, login, seuser, do_reload)
    else:
        module.fail_json(msg=f'Invalid value of argument "state": {state}')

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pipx
short_description: Manages applications installed with pipx
version_added: 3.8.0
description:
  - Manage Python applications installed in isolated virtualenvs using pipx.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._pipx
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  state:
    type: str
    choices:
      - present
      - absent
      - install
      - install_all
      - uninstall
      - uninstall_all
      - inject
      - uninject
      - upgrade
      - upgrade_shared
      - upgrade_all
      - reinstall
      - reinstall_all
      - latest
      - pin
      - unpin
    default: install
    description:
      - Desired state for the application.
      - The states V(present) and V(absent) are aliases to V(install) and V(uninstall), respectively.
      - The state V(latest) is equivalent to executing the task twice, with state V(install) and then V(upgrade). It was added
        in community.general 5.5.0.
      - The states V(install_all), V(uninject), V(upgrade_shared), V(pin) and V(unpin) are only available in C(pipx>=1.6.0),
        make sure to have a compatible version when using this option. These states have been added in community.general 9.4.0.
  name:
    type: str
    description:
      - The name of the application and also the name of the Python package being installed.
      - In C(pipx) documentation it is also referred to as the name of the virtual environment where the application is installed.
      - If O(name) is a simple package name without version specifiers, then that name is used as the Python package name
        to be installed.
      - Starting in community.general 10.7.0, you can use package specifiers when O(state=present) or O(state=install). For
        example, O(name=tox<4.0.0) or O(name=tox>3.0.27).
      - Please note that when you use O(state=present) and O(name) with version specifiers, contrary to the behavior of C(pipx),
        this module honors the version specifier and installs a version of the application that satisfies it. If you want
        to ensure the reinstallation of the application even when the version specifier is met, then you must use O(force=true),
        or perhaps use O(state=upgrade) instead.
      - Use O(source) for installing from URLs or directories.
  source:
    type: str
    description:
      - Source for the package. This option is used when O(state=install) or O(state=latest), and it is ignored with other
        states.
      - Use O(source) when installing a Python package with version specifier, or from a local path, from a VCS URL or compressed
        file.
      - The value of this option is passed as-is to C(pipx).
      - O(name) is still required when using O(source) to establish the application name without fetching the package from
        a remote source.
      - The module is not idempotent when using O(source).
  install_apps:
    description:
      - Add apps from the injected packages.
      - Only used when O(state=inject).
    type: bool
    default: false
    version_added: 6.5.0
  install_deps:
    description:
      - Include applications of dependent packages.
      - Only used when O(state=install), O(state=latest), or O(state=inject).
    type: bool
    default: false
  inject_packages:
    description:
      - Packages to be injected into an existing virtual environment.
      - Only used when O(state=inject).
    type: list
    elements: str
  force:
    description:
      - Force modification of the application's virtual environment. See C(pipx) for details.
      - Only used when O(state=install), O(state=upgrade), O(state=upgrade_all), O(state=latest), or O(state=inject).
      - The module is not idempotent when O(force=true).
    type: bool
    default: false
  include_injected:
    description:
      - Upgrade the injected packages along with the application.
      - Only used when O(state=upgrade), O(state=upgrade_all), or O(state=latest).
      - This is used with O(state=upgrade) and O(state=latest) since community.general 6.6.0.
    type: bool
    default: false
  index_url:
    description:
      - Base URL of Python Package Index.
      - Only used when O(state=install), O(state=upgrade), O(state=latest), or O(state=inject).
    type: str
  python:
    description:
      - Python version to be used when creating the application virtual environment. Must be 3.6+.
      - Only used when O(state=install), O(state=latest), O(state=reinstall), or O(state=reinstall_all).
    type: str
  system_site_packages:
    description:
      - Give application virtual environment access to the system site-packages directory.
      - Only used when O(state=install) or O(state=latest).
    type: bool
    default: false
    version_added: 6.6.0
  editable:
    description:
      - Install the project in editable mode.
    type: bool
    default: false
    version_added: 4.6.0
  pip_args:
    description:
      - Arbitrary arguments to pass directly to C(pip).
    type: str
    version_added: 4.6.0
  suffix:
    description:
      - Optional suffix for virtual environment and executable names.
      - B(Warning:) C(pipx) documentation states this is an B(experimental) feature subject to change.
    type: str
    version_added: 9.3.0
  global:
    version_added: 9.4.0
  spec_metadata:
    description:
      - Spec metadata file for O(state=install_all).
      - This content of the file is usually generated with C(pipx list --json), and it can be obtained with M(community.general.pipx_info)
        with O(community.general.pipx_info#module:include_raw=true) and obtaining the content from the RV(community.general.pipx_info#module:raw_output).
    type: path
    version_added: 9.4.0
requirements:
  - When using O(name) with version specifiers, the Python package C(packaging) is required.
  - If the package C(packaging) is at a version lesser than C(22.0.0), it fails silently when processing invalid specifiers,
    like C(tox<<<<4.0).
author:
  - "Alexei Znamensky (@russoz)"
"""

EXAMPLES = r"""
- name: Install tox
  community.general.pipx:
    name: tox

- name: Install tox from git repository
  community.general.pipx:
    name: tox
    source: git+https://github.com/tox-dev/tox.git

- name: Upgrade tox
  community.general.pipx:
    name: tox
    state: upgrade

- name: Install or upgrade tox with extra 'docs'
  community.general.pipx:
    name: tox
    source: tox[docs]
    state: latest

- name: Reinstall black with specific Python version
  community.general.pipx:
    name: black
    state: reinstall
    python: 3.7

- name: Uninstall pycowsay
  community.general.pipx:
    name: pycowsay
    state: absent

- name: Install multiple packages from list
  vars:
    pipx_packages:
      - pycowsay
      - black
      - tox
  community.general.pipx:
    name: "{{ item }}"
    state: latest
  with_items: "{{ pipx_packages }}"
"""

RETURN = r"""
version:
  description: Version of pipx.
  type: str
  returned: always
  sample: "1.7.1"
  version_added: 10.1.0
"""


from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper
from ansible_collections.community.general.plugins.module_utils._pipx import (
    make_process_dict,
    pipx_common_argspec,
    pipx_runner,
)
from ansible_collections.community.general.plugins.module_utils._pkg_req import PackageRequirement
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


def make_installed_name(name, suffix):
    return name if suffix is None else f"{name}{suffix}"


class PipX(StateModuleHelper):
    output_params = ["name", "source", "index_url", "force", "installdeps"]
    argument_spec = dict(
        state=dict(
            type="str",
            default="install",
            choices=[
                "present",
                "absent",
                "install",
                "install_all",
                "uninstall",
                "uninstall_all",
                "inject",
                "uninject",
                "upgrade",
                "upgrade_shared",
                "upgrade_all",
                "reinstall",
                "reinstall_all",
                "latest",
                "pin",
                "unpin",
            ],
        ),
        name=dict(type="str"),
        source=dict(type="str"),
        install_apps=dict(type="bool", default=False),
        install_deps=dict(type="bool", default=False),
        inject_packages=dict(type="list", elements="str"),
        force=dict(type="bool", default=False),
        include_injected=dict(type="bool", default=False),
        index_url=dict(type="str"),
        python=dict(type="str"),
        system_site_packages=dict(type="bool", default=False),
        editable=dict(type="bool", default=False),
        pip_args=dict(type="str"),
        suffix=dict(type="str"),
        spec_metadata=dict(type="path"),
    )
    argument_spec.update(pipx_common_argspec)

    module = dict(
        argument_spec=argument_spec,
        required_if=[
            ("state", "present", ["name"]),
            ("state", "install", ["name"]),
            ("state", "install_all", ["spec_metadata"]),
            ("state", "absent", ["name"]),
            ("state", "uninstall", ["name"]),
            ("state", "upgrade", ["name"]),
            ("state", "reinstall", ["name"]),
            ("state", "latest", ["name"]),
            ("state", "inject", ["name", "inject_packages"]),
            ("state", "pin", ["name"]),
            ("state", "unpin", ["name"]),
        ],
        required_by=dict(
            suffix="name",
        ),
        supports_check_mode=True,
    )

    def _retrieve_installed(self):
        output_process = make_process_dict(include_injected=True)
        installed, dummy = self.runner("_list global", output_process=output_process).run()

        if self.app_name is None:
            return installed

        return {k: v for k, v in installed.items() if k == self.app_name}

    def __init_module__(self):
        self.runner = pipx_runner(self.module, self.vars.executable)

        pkg_req = PackageRequirement(self.module, self.vars.name)
        self.parsed_name = pkg_req.parsed_name
        self.parsed_req = pkg_req.requirement
        self.app_name = make_installed_name(self.parsed_name, self.vars.suffix)

        self.vars.set("application", self._retrieve_installed(), change=True, diff=True)

        with self.runner("version") as ctx:
            rc, out, err = ctx.run()
            self.vars.version = out.strip()

        if LooseVersion(self.vars.version) < LooseVersion("1.7.0"):
            self.do_raise("The pipx tool must be at least at version 1.7.0")

    def __quit_module__(self):
        self.vars.application = self._retrieve_installed()

    def _capture_results(self, ctx):
        self.vars.stdout = ctx.results_out
        self.vars.stderr = ctx.results_err
        self.vars.cmd = ctx.cmd
        self.vars.set("run_info", ctx.run_info, verbosity=4)

    def state_install(self):
        # If we have a version spec and no source, use the version spec as source
        if self.parsed_req and not self.vars.source:
            self.vars.source = self.vars.name

        if self.vars.application.get(self.app_name):
            is_installed = True
            version_match = (
                self.vars.application[self.app_name]["version"] in self.parsed_req.specifier
                if self.parsed_req
                else True
            )
            force = self.vars.force or (not version_match)
        else:
            is_installed = False
            version_match = False
            force = self.vars.force

        if is_installed and version_match and not force:
            return

        self.changed = True
        args_order = (
            "state global index_url install_deps force python system_site_packages editable pip_args suffix name_source"
        )
        with self.runner(args_order, check_mode_skip=True) as ctx:
            ctx.run(name_source=[self.parsed_name, self.vars.source], force=force)
            self._capture_results(ctx)

    state_present = state_install

    def state_install_all(self):
        self.changed = True
        with self.runner(
            "state global index_url force python system_site_packages editable pip_args spec_metadata",
            check_mode_skip=True,
        ) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_upgrade(self):
        name = make_installed_name(self.vars.name, self.vars.suffix)
        if not self.vars.application:
            self.do_raise(f"Trying to upgrade a non-existent application: {name}")
        if self.vars.force:
            self.changed = True

        with self.runner(
            "state global include_injected index_url force editable pip_args name", check_mode_skip=True
        ) as ctx:
            ctx.run(name=name)
            self._capture_results(ctx)

    def state_uninstall(self):
        if self.vars.application:
            name = make_installed_name(self.vars.name, self.vars.suffix)
            with self.runner("state global name", check_mode_skip=True) as ctx:
                ctx.run(name=name)
                self._capture_results(ctx)

    state_absent = state_uninstall

    def state_reinstall(self):
        name = make_installed_name(self.vars.name, self.vars.suffix)
        if not self.vars.application:
            self.do_raise(f"Trying to reinstall a non-existent application: {name}")
        self.changed = True
        with self.runner("state global name python", check_mode_skip=True) as ctx:
            ctx.run(name=name)
            self._capture_results(ctx)

    def state_inject(self):
        name = make_installed_name(self.vars.name, self.vars.suffix)
        if not self.vars.application:
            self.do_raise(f"Trying to inject packages into a non-existent application: {name}")
        if self.vars.force:
            self.changed = True
        with self.runner(
            "state global index_url install_apps install_deps force editable pip_args name inject_packages",
            check_mode_skip=True,
        ) as ctx:
            ctx.run(name=name)
            self._capture_results(ctx)

    def state_uninject(self):
        name = make_installed_name(self.vars.name, self.vars.suffix)
        if not self.vars.application:
            self.do_raise(f"Trying to uninject packages into a non-existent application: {name}")
        with self.runner("state global name inject_packages", check_mode_skip=True) as ctx:
            ctx.run(name=name)
            self._capture_results(ctx)

    def state_uninstall_all(self):
        with self.runner("state global", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_reinstall_all(self):
        with self.runner("state global python", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_upgrade_all(self):
        if self.vars.force:
            self.changed = True
        with self.runner("state global include_injected force", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_upgrade_shared(self):
        with self.runner("state global pip_args", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_latest(self):
        if not self.vars.application or self.vars.force:
            self.changed = True
            args_order = "state global index_url install_deps force python system_site_packages editable pip_args suffix name_source"
            with self.runner(args_order, check_mode_skip=True) as ctx:
                ctx.run(state="install", name_source=[self.vars.name, self.vars.source])
                self._capture_results(ctx)

        with self.runner(
            "state global include_injected index_url force editable pip_args name", check_mode_skip=True
        ) as ctx:
            ctx.run(state="upgrade")
            self._capture_results(ctx)

    def state_pin(self):
        with self.runner("state global name", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)

    def state_unpin(self):
        with self.runner("state global name", check_mode_skip=True) as ctx:
            ctx.run()
            self._capture_results(ctx)


def main():
    PipX.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: xattr
short_description: Manage user defined extended attributes
description:
  - Manages filesystem user defined extended attributes.
  - Requires that extended attributes are enabled on the target filesystem and that the C(setfattr)/C(getfattr) utilities
    are present.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  path:
    description:
      - The full path of the file/object to get the facts of.
    type: path
    required: true
    aliases: [name]
  namespace:
    description:
      - Namespace of the named name/key.
    type: str
    default: user
  key:
    description:
      - The name of a specific Extended attribute key to set/retrieve.
    type: str
  value:
    description:
      - The value to set the named name/key to, it automatically sets the O(state) to V(present).
    type: str
  state:
    description:
      - Defines which state you want to do.
      - V(read) retrieves the current value for a O(key).
      - V(present) sets O(path) to O(value), default if value is set.
      - V(all) dumps all data.
      - V(keys) retrieves all keys.
      - V(absent) deletes the key.
    type: str
    choices: [absent, all, keys, present, read]
    default: read
  follow:
    description:
      - If V(true), dereferences symlinks and sets/gets attributes on symlink target, otherwise acts on symlink itself.
    type: bool
    default: true
author:
  - Brian Coca (@bcoca)
"""

EXAMPLES = r"""
- name: Obtain the extended attributes of /etc/foo.conf
  community.general.xattr:
    path: /etc/foo.conf

- name: Set the key 'user.foo' to value 'bar'
  community.general.xattr:
    path: /etc/foo.conf
    key: foo
    value: bar

- name: Set the key 'trusted.glusterfs.volume-id' to value '0x817b94343f164f199e5b573b4ea1f914'
  community.general.xattr:
    path: /mnt/bricks/brick1
    namespace: trusted
    key: glusterfs.volume-id
    value: "0x817b94343f164f199e5b573b4ea1f914"

- name: Remove the key 'user.foo'
  community.general.xattr:
    path: /etc/foo.conf
    key: foo
    state: absent

- name: Remove the key 'trusted.glusterfs.volume-id'
  community.general.xattr:
    path: /mnt/bricks/brick1
    namespace: trusted
    key: glusterfs.volume-id
    state: absent
"""

import os

# import module snippets
from ansible.module_utils.basic import AnsibleModule


def get_xattr_keys(module, path, follow):
    cmd = [module.get_bin_path("getfattr", True), "--absolute-names"]

    if not follow:
        cmd.append("-h")
    cmd.append(path)

    return _run_xattr(module, cmd)


def get_xattr(module, path, key, follow):
    cmd = [module.get_bin_path("getfattr", True), "--absolute-names"]

    if not follow:
        cmd.append("-h")
    if key is None:
        cmd.append("-d")
    else:
        cmd.append("-n")
        cmd.append(key)
    cmd.append(path)

    return _run_xattr(module, cmd, False)


def set_xattr(module, path, key, value, follow):
    cmd = [module.get_bin_path("setfattr", True)]
    if not follow:
        cmd.append("-h")
    cmd.append("-n")
    cmd.append(key)
    cmd.append("-v")
    cmd.append(value)
    cmd.append(path)

    return _run_xattr(module, cmd)


def rm_xattr(module, path, key, follow):
    cmd = [module.get_bin_path("setfattr", True)]
    if not follow:
        cmd.append("-h")
    cmd.append("-x")
    cmd.append(key)
    cmd.append(path)

    return _run_xattr(module, cmd, False)


def _run_xattr(module, cmd, check_rc=True):
    try:
        (rc, out, err) = module.run_command(cmd, check_rc=check_rc)
    except Exception as e:
        module.fail_json(msg=f"{e}!")

    # result = {'raw': out}
    result = {}
    for line in out.splitlines():
        if line.startswith("#") or line == "":
            pass
        elif "=" in line:
            (key, val) = line.split("=", 1)
            result[key] = val.strip('"')
        else:
            result[line] = ""
    return result


def main():
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type="path", required=True, aliases=["name"]),
            namespace=dict(type="str", default="user"),
            key=dict(type="str", no_log=False),
            value=dict(type="str"),
            state=dict(type="str", default="read", choices=["absent", "all", "keys", "present", "read"]),
            follow=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
    path = module.params.get("path")
    namespace = module.params.get("namespace")
    key = module.params.get("key")
    value = module.params.get("value")
    state = module.params.get("state")
    follow = module.params.get("follow")

    if not os.path.exists(path):
        module.fail_json(msg="path not found or not accessible!")

    changed = False
    msg = ""
    res = {}

    if key is None and state in ["absent", "present"]:
        module.fail_json(msg=f"{state} needs a key parameter")

    # Prepend the key with the namespace if defined
    if (
        key is not None
        and namespace is not None
        and len(namespace) > 0
        and not (namespace == "user" and key.startswith("user."))
    ):
        key = f"{namespace}.{key}"

    if state == "present" or value is not None:
        current = get_xattr(module, path, key, follow)
        if current is None or key not in current or value != current[key]:
            if not module.check_mode:
                res = set_xattr(module, path, key, value, follow)
            changed = True
        res = current
        msg = f"{key} set to {value}"
    elif state == "absent":
        current = get_xattr(module, path, key, follow)
        if current is not None and key in current:
            if not module.check_mode:
                res = rm_xattr(module, path, key, follow)
            changed = True
        res = current
        msg = f"{key} removed"
    elif state == "keys":
        res = get_xattr_keys(module, path, follow)
        msg = "returning all keys"
    elif state == "all":
        res = get_xattr(module, path, None, follow)
        msg = "dumping all"
    else:
        res = get_xattr(module, path, key, follow)
        msg = f"returning {key}"

    module.exit_json(changed=changed, msg=msg, xattr=res)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Adam Števko <adam.stevko@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: zpool_facts
short_description: Gather facts about ZFS pools
description:
  - Gather facts from ZFS pool properties.
author: Adam Števko (@xen0l)
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.facts
  - community.general._attributes.facts_module
options:
  name:
    description:
      - ZFS pool name.
    type: str
    aliases: ["pool", "zpool"]
  parsable:
    description:
      - Specifies if property values should be displayed in machine friendly format.
    type: bool
    default: false
  properties:
    description:
      - Specifies which dataset properties should be queried in comma-separated format. For more information about dataset
        properties, check zpool(1M) man page.
    type: str
    default: all
"""

EXAMPLES = r"""
- name: Gather facts about ZFS pool rpool
  community.general.zpool_facts: pool=rpool

- name: Gather space usage about all imported ZFS pools
  community.general.zpool_facts: properties='free,size'

- name: Print gathered information
  ansible.builtin.debug:
    msg: 'ZFS pool {{ item.name }} has {{ item.free }} free space out of {{ item.size }}.'
  with_items: '{{ ansible_zfs_pools }}'
"""

RETURN = r"""
ansible_facts:
  description: Dictionary containing all the detailed information about the ZFS pool facts.
  returned: success
  type: dict
  contains:
    ansible_zfs_pools:
      description: ZFS pool facts.
      returned: success
      type: list
      elements: dict
      sample:
        - "allocated": "3.46G"
          "altroot": "-"
          "autoexpand": "off"
          "autoreplace": "off"
          "bootfs": "rpool/ROOT/openindiana"
          "cachefile": "-"
          "capacity": "6%"
          "comment": "-"
          "dedupditto": "0"
          "dedupratio": "1.00x"
          "delegation": "on"
          "expandsize": "-"
          "failmode": "wait"
          "feature@async_destroy": "enabled"
          "feature@bookmarks": "enabled"
          "feature@edonr": "enabled"
          "feature@embedded_data": "active"
          "feature@empty_bpobj": "active"
          "feature@enabled_txg": "active"
          "feature@extensible_dataset": "enabled"
          "feature@filesystem_limits": "enabled"
          "feature@hole_birth": "active"
          "feature@large_blocks": "enabled"
          "feature@lz4_compress": "active"
          "feature@multi_vdev_crash_dump": "enabled"
          "feature@sha512": "enabled"
          "feature@skein": "enabled"
          "feature@spacemap_histogram": "active"
          "fragmentation": "3%"
          "free": "46.3G"
          "freeing": "0"
          "guid": "15729052870819522408"
          "health": "ONLINE"
          "leaked": "0"
          "listsnapshots": "off"
          "name": "rpool"
          "readonly": "off"
          "size": "49.8G"
          "version": "-"
name:
  description: ZFS pool name.
  returned: success
  type: str
  sample: rpool
parsable:
  description: If parsable output should be provided in machine friendly format.
  returned: if O(parsable=true)
  type: bool
  sample: true
"""

from collections import defaultdict

from ansible.module_utils.basic import AnsibleModule


class ZPoolFacts:
    def __init__(self, module):
        self.module = module
        self.name = module.params["name"]
        self.parsable = module.params["parsable"]
        self.properties = module.params["properties"]
        self._pools = defaultdict(dict)
        self.facts = []

    def pool_exists(self):
        cmd = [self.module.get_bin_path("zpool"), "list", self.name]
        rc, dummy, dummy = self.module.run_command(cmd)
        return rc == 0

    def get_facts(self):
        cmd = [self.module.get_bin_path("zpool"), "get", "-H"]
        if self.parsable:
            cmd.append("-p")
        cmd.append("-o")
        cmd.append("name,property,value")
        cmd.append(self.properties)
        if self.name:
            cmd.append(self.name)

        rc, out, err = self.module.run_command(cmd, check_rc=True)

        for line in out.splitlines():
            pool, prop, value = line.split("\t")

            self._pools[pool].update({prop: value})

        for k, v in self._pools.items():
            v.update({"name": k})
            self.facts.append(v)

        return {"ansible_zfs_pools": self.facts}


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(aliases=["pool", "zpool"], type="str"),
            parsable=dict(default=False, type="bool"),
            properties=dict(default="all", type="str"),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    zpool_facts = ZPoolFacts(module)

    result = {
        "changed": False,
        "name": zpool_facts.name,
    }
    if zpool_facts.parsable:
        result["parsable"] = zpool_facts.parsable

    if zpool_facts.name is not None:
        if zpool_facts.pool_exists():
            result["ansible_facts"] = zpool_facts.get_facts()
        else:
            module.fail_json(msg=f"ZFS pool {zpool_facts.name} does not exist!")
    else:
        result["ansible_facts"] = zpool_facts.get_facts()

    module.exit_json(**result)


if __name__ == "__main__":
    main()
